#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import logging
import h5py
import numpy as np

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

from . import ccmap as cmp
from . import ccmapHelpers as cmh


class GCMAP:
    """To access Genome wide contact map.

    It is similar to :class:`gcMapExplorer.lib.ccmap.CCMAP` and contains same attributes.
    Therefore, both :class:`gcMapExplorer.lib.ccmap.CCMAP` and ``GCMAP`` can be used in same way to accesss attributes.
    It also contains additional attributes because it uses HDF5 file to read the maps on demand.

    Structure of gcmap file:

        ::

            HDF5
            │
            ├──────── chr1 ──── Attributes : ['xlabel', 'ylabel', 'compression']
            │           │
            │           ├────── 10kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 20kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 40kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 60kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 80kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 160kb ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 320kb ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 640kb ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           │
            │           ├────── 10kb-bNoData  ( 1D Numpy Array )
            │           ├────── 20kb-bNoData  ( 1D Numpy Array )
            │           ├────── 40kb-bNoData  ( 1D Numpy Array )
            │           ├────── 60kb-bNoData  ( 1D Numpy Array )
            │           ├────── 80kb-bNoData  ( 1D Numpy Array )
            │           ├────── 160kb-bNoData ( 1D Numpy Array )
            │           ├────── 320kb-bNoData ( 1D Numpy Array )
            │           └────── 640kb-bNoData ( 1D Numpy Array )
            │
            ├──────── chr2 ──── Attributes : ['xlabel', 'ylabel', 'compression']
            │           │
            │           ├────── 10kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 20kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 40kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 60kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 80kb  ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 160kb ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 320kb ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           ├────── 640kb ( 2D Numpy Array ) ─── Attributes : ['minvalue', 'maxvalue', 'xshape', 'yshape', 'binsize']
            │           │
            │           ├────── 10kb-bNoData  ( 1D Numpy Array )
            │           ├────── 20kb-bNoData  ( 1D Numpy Array )
            │           ├────── 40kb-bNoData  ( 1D Numpy Array )
            │           ├────── 60kb-bNoData  ( 1D Numpy Array )
            │           ├────── 80kb-bNoData  ( 1D Numpy Array )
            │           ├────── 160kb-bNoData ( 1D Numpy Array )
            │           ├────── 320kb-bNoData ( 1D Numpy Array )
            │           └────── 640kb-bNoData ( 1D Numpy Array )
            :
            :
            :
            └───── ...


    .. note::
        * Reading the entire map from HDF5 could be time taking for very large map. Therefore, use this class in cases like read small region of map or read only once.
        * To perform calculation, use :meth:`gcMapExplorer.lib.gcmap.loadGCMapAsCCMap` as it returns :class:`gcMapExplorer.lib.ccmap.CCMAP`.

    The class is instantiated by two methods:
        >>> ccMapObj = gcMapExplorer.lib.ccmap.CCMAP(hdf5, 'chr22')    # To read chr22 vs chr22 map
        >>> ccMapObj.matrix[200:400, 200:400]  # Read region between 200 to 400 of chr22 vs chr22 map.


    Parameters
    ----------

    hdf5 : str or h5py.File
        Either gcmap file name or h5py file object, which is an entry point to HDF5 file.
    mapName : str
        Name of map. It could be chromosome name in case of intra-chromosomal map.
        e.g.: ``chr1`` or ``chr2``.
    chromAtX : str
        chromosome at X-axis. In case of intra-chromosomal map, this is not required because
        both at X and Y axis same chromosome is present.
    chromAtX : str
        chromosome at Y-axis. In case of intra-chromosomal map, this is not required because
        both at X and Y axis same chromosome is present. If ``chromAtY = None``, both x-axis
        and y-axis contains same chromosome and map is of 'intra' of 'cis' type.
    resolution : str
        Input resolution to read from file.

    Attributes
    ==========
    yticks      :   list
        Minimum and maximum locations along Y-axis. e.g. ``yticks=[0, 400000]``
    xticks      :   list
        Minimum and maximum locations along X-axis. e.g. ``xticks=[0, 400000]``
    binsize     :  int
        Resolution of data. In case of 10kb resolution, binsize is 10000.
    title       :   str
        Title of the data
    xlabel      :   str
        Title for X-axis, which is chromosome name along X-axis
    ylabel      :   str
        Title for Y-axis, which is chromosome name along Y-axis
    shape       :   tuple
        Overall shape of matrix
    minvalue    :   float
        Minimum value in matrix
    maxvalue    :   float
        Maximum value in matrix
    matrix      :   h5py.Dataset
        A HDF5 Dataset object pointing to matrix/map. `See here for more details: <http://docs.h5py.org/en/latest/high/dataset.html#dataset>`_
    bNoData     :   numpy.ndarray
        A boolean numpy array of matrix shape
    bLog        :   bool
        If values in matrix are in log
    dtype       :   str
        Data type of matrix/map
    mapType     :   str
        Type of map: ``intra`` or ``inter`` chromosomal map. If chromosome along X- and Y- axis is same, then map is intra-chromosomal,
        otherwise map is inter-chromosomal.
    hdf5        :   h5py.File
        HDF5 file object instance
    fileOpened  : bool
        Whether a file is opened inside object or a HDF5 file object is provided to object.
        When a file is opened by object, it is closed before object is destroyed.
    groupName   :   str
        Name of current contact map or group name in HDF5 file
    resolution  :   str
        Resolution of current contact map
    finestResolution    :   str
        Finest available resolution of current contact map
    binsizes    :   list
        List of binsizes available for current contact map
    mapNameList :   list
        List of all available contact maps in gcmap file


    """

    def __init__(self, hdf5, mapName=None, chromAtX=None, chromAtY=None, resolution=None):
        self.xticks = None
        self.yticks = None
        self.binsize = 0
        self.title = None
        self.xlabel = None
        self.ylabel = None
        self.shape = None
        self.bNoData = None
        self.minvalue = 9e10
        self.maxvalue = -9e10
        self.bLog = False
        self.dtype = None
        self.matrix = None
        self.mapType = None

        self.hdf5 = None
        self.fileOpened = False
        self.groupName = None
        self.resolution = None
        self.finestResolution = None
        self.binsizes = None
        self.mapNameList = None

        if isinstance(hdf5, str):
            self.hdf5 = h5py.File(hdf5)
            self.fileOpened = True
        elif isinstance(hdf5, h5py.File):
            self.hdf5 = hdf5
        else:
            raise TypeError ('{0} is not a gcmap file or h5py.File instance'.format(hdf5))

        # In case if map name is not given, load the first map
        if mapName is None and chromAtX is None and chromAtY is None:
            self.genMapNameList()
            mapName = self.mapNameList[0]

        self._setLabels(mapName, chromAtX, chromAtY)
        self._readMap(resolution=resolution)

    def __del__(self):
        if self.fileOpened:
            self.hdf5.close()

    def genMapNameList(self, sortBy='name'):
        """ Generate list of contact maps available in gcmap file

        The maps can be either sorted by name or by size.
        The listed maps are in ``GCMAP.mapNameList``.

        Parameters
        ----------
        sortBy : str
            Accepted keywords are ``name`` and ``size``.

        """
        mapList = []
        for mapName in self.hdf5.keys():
            mapList.append(mapName)

        if sortBy == 'size':
            # Get mapname and resolution of already loaded map
            oldMapName = None
            oldResolution = None
            if self.groupName is not None:
                oldMapName = self.groupName
                oldResolution = self.resolution

            # Generate list of shapes
            shapes = []
            for mapName in mapList:
                self.changeMap(mapName=mapName)
                shapes.append(max(self.shape[0], self.shape[1]))

            # Sort and store
            idx = np.argsort(shapes)
            self.mapNameList = list( np.array(mapList)[idx] )

            # Revert back to original map
            if oldMapName is not None:
                self.changeMap(mapName=oldMapName, resolution=oldResolution)

        else:
            self.mapNameList = cmh.sorted_nicely( mapList )

    def _setLabels(self, mapName, chromAtX, chromAtY):
        """ Set xlabel and ylabel
        """

        if mapName is not None:
            if mapName not in self.hdf5:
                raise cmp.MapNotFoundError(' [{0}] dataset not found in [{1}] file...'.format(mapName, self.hdf5.filename))
            self.groupName = mapName
            self.xlabel = self.hdf5[mapName].attrs['xlabel']
            self.ylabel = self.hdf5[mapName].attrs['ylabel']
        else:
            if chromAtX is not None:
                self.xlabel = chromAtX
                if chromAtY is not None:
                    self.ylabel = chromAtY
                else:
                    self.ylabel = self.xlabel
            elif chromAtY is not None:
                self.ylabel = chromAtY
                if chromAtX is not None:
                    self.xlabel = chromAtX
                else:
                    self.xlabel = self.ylabel
            else:
                raise AssertionError('Either mapName or chromAtX or chromAtY should be provided for further processing.')

            self.groupName = None
            if self.xlabel == self.ylabel:
                self.groupName = self.xlabel
            else:
                self.groupName = self.xlabel + '-' + self.ylabel

        self.mapType = None
        if self.xlabel == self.ylabel:
            self.mapType = 'intra'
        else:
            self.mapType = 'inter'

    def _readMap(self, resolution=None):
        """ Temporarily store h5py.Dataset object to GCMAP.matrix and update all attributes for given map.
        """

        if self.groupName not in self.hdf5:
            raise cmp.MapNotFoundError(' [{0}] dataset not found in [{1}] file...'.format(self.groupName, self.hdf5.filename))

        # determining finest resolution map
        self.binsizes = []
        for key in self.hdf5[self.groupName].keys():
            if 'bNoData' not in key:
                self.binsizes.append( self.hdf5[self.groupName][key].attrs['binsize'] )
        self.binsizes = sorted(self.binsizes)

        # At the start, always choose finest resolution
        self.finestResolution = cmp.binsizeToResolution(self.binsizes[0])
        if resolution is not None:
            resolutionList = list( map(cmp.binsizeToResolution, self.binsizes ) )
            if resolution in resolutionList:
                self.resolution = resolution
            else:
                raise cmp.ResolutionNotFoundError (' "{0}" resolution not found for "{1}" in file: "{2}".'.format(resolution, self.groupName, self.hdf5.filename ) )
        else:
            self.resolution = self.finestResolution

        self.dtype = self.hdf5[self.groupName][self.resolution].dtype

        for key in ['xlabel', 'ylabel']:
            self.__dict__[key] = self.hdf5[self.groupName].attrs[key]

        for key in ['minvalue', 'maxvalue', 'binsize']:
            self.__dict__[key] = self.hdf5[self.groupName][self.resolution].attrs[key]

        self.shape = (self.hdf5[self.groupName][self.resolution].attrs['xshape'], self.hdf5[self.groupName][self.resolution].attrs['yshape'])
        self.xticks = [0, self.shape[0]*self.binsize]
        self.yticks = [0, self.shape[1]*self.binsize]

        self.title = self.xlabel + '_vs_' + self.ylabel

        if self.resolution+'-bNoData' in self.hdf5[self.groupName]:
            self.bNoData = np.asarray( self.hdf5[self.groupName][self.resolution+'-bNoData'][:], dtype=np.bool )
        self.matrix = self.hdf5[self.groupName][self.resolution]

    def loadSmallestMap(self, resolution=None):
        """ Load smallest sized contact map

        Parameters
        ----------
        resolution : str
            Resolution to change. When it is not provided, finest resolution of smallest maps is loaded.


        """
        self.genMapNameList(sortBy='size')
        self.changeMap(mapName=self.mapNameList[0], resolution=resolution)

    def toCoarserResolution(self):
        """ Try to change contact map to next coarser resolution

        Returns
        -------
        success : bool
            If change was successfu ``True`` otherwise ``False``.

        """

        idx = self.binsizes.index(self.binsize)
        if idx == len(self.binsizes) - 1:
            return False

        self.resolution = cmp.binsizeToResolution(self.binsizes[idx+1])

        self.dtype = self.hdf5[self.groupName][self.resolution].dtype

        for key in ['minvalue', 'maxvalue', 'binsize']:
            self.__dict__[key] = self.hdf5[self.groupName][self.resolution].attrs[key]

        self.shape = (self.hdf5[self.groupName][self.resolution].attrs['xshape'], self.hdf5[self.groupName][self.resolution].attrs['yshape'])
        self.xticks = [0, self.shape[0]*self.binsize]
        self.yticks = [0, self.shape[1]*self.binsize]

        self.title = self.xlabel + '_vs_' + self.ylabel

        if self.resolution+'-bNoData' in self.hdf5[self.groupName]:
            self.bNoData = np.asarray( self.hdf5[self.groupName][self.resolution+'-bNoData'][:], dtype=np.bool )
        self.matrix = self.hdf5[self.groupName][self.resolution]

        return True

    def toFinerResolution(self):
        """ Try to change contact map to next finer resolution

        Returns
        -------
        success : bool
            If change was successfu ``True`` otherwise ``False``.

        """

        idx = self.binsizes.index(self.binsize)
        if idx == 0:
            return False

        self.resolution = cmp.binsizeToResolution(self.binsizes[idx-1])

        self.dtype = self.hdf5[self.groupName][self.resolution].dtype

        for key in ['minvalue', 'maxvalue', 'binsize']:
            self.__dict__[key] = self.hdf5[self.groupName][self.resolution].attrs[key]

        self.shape = (self.hdf5[self.groupName][self.resolution].attrs['xshape'], self.hdf5[self.groupName][self.resolution].attrs['yshape'])
        self.xticks = [0, self.shape[0]*self.binsize]
        self.yticks = [0, self.shape[1]*self.binsize]

        self.title = self.xlabel + '_vs_' + self.ylabel

        if self.resolution+'-bNoData' in self.hdf5[self.groupName]:
            self.bNoData = np.asarray( self.hdf5[self.groupName][self.resolution+'-bNoData'][:], dtype=np.bool )
        self.matrix = self.hdf5[self.groupName][self.resolution]

        return True

    def changeResolution(self, resolution):
        """ Try to change contact map of a given resolution.

        Parameters
        ----------
        resolution : str
            Resolution to change.

        Returns
        -------
        success : bool
            If change was successfu ``True`` otherwise ``False``.

        """

        self._readMap(resolution=resolution)

    def changeMap(self, mapName=None, chromAtX=None, chromAtY=None, resolution=None):
        """ Change the map for another chromosome

        It can be used to change the map. For example, to access the map of 'chr20' instead of 'chr22', use this function.

        Parameters
        ----------
        mapName : str
            Name of map. It could be chromosome name in case of intra-chromosomal map.
            e.g.: ``chr1`` or ``chr2``.
        chromAtX : str
            chromosome at X-axis. In case of intra-chromosomal map, this is not required because
            both at X and Y axis same chromosome is present.
        chromAtX : str
            chromosome at Y-axis. In case of intra-chromosomal map, this is not required because
            both at X and Y axis same chromosome is present. If ``chromAtY = None``, both x-axis
            and y-axis contains same chromosome and map is of 'intra' of 'cis' type.
        resolution : str
            Input resolution to read from file.


        For example:
            >>> ccMapObj = gcMapExplorer.lib.ccmap.CCMAP(hdf5, 'chr22')    # To read chr22 vs chr22 map
            >>> ccMapObj.matrix[200:400, 200:400]  # To access region between 200 to 400 of chr22 vs chr22 map.
            >>> ccMapObj.changeMap('chr20')        # Changed to read chr20 vs chr20 map
            >>> ccMapObj.matrix[200:400, 200:400]  # Now, to access region between 200 to 400 of chr20 vs chr20 map.


        """

        self._setLabels(mapName, chromAtX, chromAtY)
        self.matrix = None
        self._readMap(resolution=resolution)

    def checkMapExist(self, mapName=None, chromAtX=None, chromAtY=None, resolution=None):
        """ Check if a map is exist in the file

        It can be used to check if a map is exist in the file.

        Parameters
        ----------
        mapName : str
            Name of map. It could be chromosome name in case of intra-chromosomal map.
            e.g.: ``chr1`` or ``chr2``.
        chromAtX : str
            chromosome at X-axis. In case of intra-chromosomal map, this is not required because
            both at X and Y axis same chromosome is present.
        chromAtX : str
            chromosome at Y-axis. In case of intra-chromosomal map, this is not required because
            both at X and Y axis same chromosome is present. If ``chromAtY = None``, both x-axis
            and y-axis contains same chromosome and map is of 'intra' of 'cis' type.
        resolution : str
            Input resolution to read from file.


        Returns
        -------
        doExist : bool
            If map is present then ``True`` otherwise ``False``.

        """

        oldMapName = self.groupName
        oldChromAtX = self.xlabel
        oldChromAtY = self.xlabel
        oldResolution = self.resolution

        doExist = True

        # try to change the resolution, and catch the error
        try:
            self._setLabels(mapName, chromAtX, chromAtY)
            self.matrix = None
            self._readMap(resolution=resolution)
        except(cmp.ResolutionNotFoundError, cmp.ResolutionNotFoundError) as e:
            doExist = False
        except Exception as e:
            raise e

        # Revert to old one
        self.changeMap(mapName=oldMapName, chromAtX=oldChromAtX, chromAtY=oldChromAtY, resolution=oldResolution)

        return doExist

    def get_ticks(self, binsize=None):
        """To get xticks and yticks for the matrix

        Parameters
        ----------
        binsize : int
            Number of base in each bin or pixel or box of contact map.

        Returns
        -------
        xticks  :   numpy.array
            1D array containing positions along X-axis
        yticks  :   numpy.array
            1D array containing positions along X-axis

        """
        if binsize is None:
            xticks = np.arange(self.xticks[0], self.xticks[1], self.binsize)
            yticks = np.arange(self.yticks[0], self.yticks[1], self.binsize)
        else:
            xticks = np.arange(self.xticks[0], self.xticks[1], binsize)
            yticks = np.arange(self.yticks[0], self.yticks[1], binsize)

        return xticks, yticks

    def _performDownSampling(self , level=2, method='sum'):

        # To determine output shape but it does not contain remainder regions
        # subtracted one as real data start from first index
        outShapeX = int( np.ceil(float(self.shape[1] - 1 )/level) )
        outShapeY = int( np.ceil(float(self.shape[0] - 1 )/level) )


        # To determine remainder regions(sourceMasked.max() - sourceMasked.min())
        pad_size_outer = ( outShapeY * level) - (self.shape[0] - 1)
        pad_size_inner = ( outShapeX * level) - (self.shape[1] - 1)

        # Otput shape
        outputShape = (outShapeX + 1 , outShapeY + 1)

        # New binsize
        newBinsize = self.binsize * level

        # Create and add map
        resolution = cmp.binsizeToResolution(newBinsize)

        if  self.hdf5[self.groupName].attrs['compression'] == 'lzf':
            outCmap = self.hdf5[self.groupName].create_dataset(resolution, outputShape, dtype=self.dtype, chunks=True, compression="lzf", shuffle=True)
        else:
            outCmap = self.hdf5[self.groupName].create_dataset(resolution, outputShape, dtype=self.dtype, chunks=True, compression="gzip", shuffle=True, compression_opts=4)

        try:
            # Ignore first row and column --- no data
            count = 1
            i = 1
            while i < self.shape[0]:

                # Here handles inner
                if pad_size_inner != 0:
                    zero_part = []
                    for n in range(level):
                        zero_part.append( np.zeros(pad_size_inner) )

                    # Here handles outer
                    if i+level < self.shape[0]:
                        x_padded = np.hstack( (self.matrix[i:i + level, 1:], zero_part) )
                    else:
                        x_padded = np.hstack( (self.matrix[i:, 1:], zero_part[:self.matrix[i:].shape[0]] ) )

                    # print(x_padded.shape)

                    if method == 'mean':
                        r = x_padded.mean(axis=0).reshape(-1, level).mean(axis=1)
                    elif method == 'max':
                        r = x_padded.max(axis=0).reshape(-1, level).max(axis=1)
                    else:
                        r = x_padded.sum(axis=0).reshape(-1, level).sum(axis=1)

                    #rint(r.shape)

                else:

                    # Here handles outer
                    if i+level < self.shape[0]:
                        if method == 'mean':
                            r = self.matrix[i:i+level, 1:].mean(axis=0).reshape(-1, level).mean(axis=1)
                        elif method == 'max':
                            r = self.matrix[i:i+level, 1:].max(axis=0).reshape(-1, level).max(axis=1)
                        else:
                            r = self.matrix[i:i+level, 1:].sum(axis=0).reshape(-1, level).sum(axis=1)
                    else:
                        remain = self.shape[0]-i
                        if method == 'mean':
                            r = self.matrix[i:, 1:].mean(axis=0).reshape(-1, remain).mean(axis=1)
                        elif method == 'max':
                            r = self.matrix[i:, 1:].max(axis=0).reshape(-1, remain).max(axis=1)
                        else:
                            r = self.matrix[i:, 1:].sum(axis=0).reshape(-1, remain).sum(axis=1)

                i = i + level

                outCmap[count, 1:] = r
                count = count + 1


            # Get minimum value othar than zero
            ma = np.ma.masked_equal(outCmap, 0.0, copy=False)
            minvalue = ma.min()
            del ma

            # Save all other attributes
            if self.bNoData is not None:
                bNoData = np.zeros(outputShape[0], dtype=np.bool)
                if pad_size_inner != 0:
                    x_padded = np.hstack( (self.bNoData[1:], np.zeros(pad_size_inner)) )
                else:
                    x_padded = self.bNoData[1:]

                bNoData[1:] = x_padded.reshape(-1, level).sum(axis=1)

                if  self.hdf5[self.groupName].attrs['compression'] == 'lzf':
                    self.hdf5[self.groupName].create_dataset(resolution+'-bNoData', bNoData.shape, dtype=bNoData.dtype, data=bNoData, chunks=True, compression="lzf", shuffle=True)
                else:
                    self.hdf5[self.groupName].create_dataset(resolution+'-bNoData', bNoData.shape, dtype=bNoData.dtype, data=bNoData, chunks=True, compression="gzip", shuffle=True, compression_opts=4)

            outCmap.attrs['minvalue'] = minvalue
            outCmap.attrs['maxvalue'] = np.amax(outCmap)
            outCmap.attrs['binsize'] = newBinsize
            outCmap.attrs['xshape'] = outputShape[0]
            outCmap.attrs['yshape'] = outputShape[1]

        except:
            # map might be incomplete, remove it
            if resolution in self.hdf5[self.groupName]:
                self.hdf5[self.groupName].pop(resolution)
            if resolution+'-bNoData' in self.hdf5[self.groupName]:
                self.hdf5[self.groupName].pop(resolution+'-bNoData')
            raise

        return True

    def performDownSampling(self, method='sum'):
        """Downsample recursively and store the maps

        It Downsample the maps and automatically add it to same input ``gcmap`` file.
        Downsampling works recursively, and downsampled maps are generated untill map has a size of less than 500.

        Parameters
        ----------
        method : str
            Method of downsampling. Three accepted methods are ``sum``: sum all values, ``mean``: Average of all values
            and ``max``: Maximum of all values.

        """

        allowed_methods= ['sum', 'mean', 'max']
        if method not in allowed_methods:
            raise ValueError(' "{0}" is not a valid keyword to downsample. Use: "sum", "mean" or "max". '.format(method))

        while self.shape[0] > 500 or self.shape[1] > 500:
            if not self._performDownSampling(method=method):
                break
            self.binsizes.append(self.binsize*2)
            self.toCoarserResolution()


def loadGCMapAsCCMap(filename, mapName=None, chromAtX=None, chromAtY=None, resolution=None, workDir=None):
    """ Load a map from gcmap file as a :class:`gcMapExplorer.lib.ccmap.CCMAP`.

    Parameters
    ----------
    filename : str
        Either a gcmap file or h5py.File instance or GCMAP.hdf5 from which contact map data will be read.
    mapName : str
        Name of contact map. e.g.: ``chr1`` or ``chr2``.
    chromAtX : str
        Name of chromosome at X-axis
    chromAtY : str
        Name of chromosome at Y-axis. If ``chromAtY = None``, both x-axis and y-axis contains same chromosome and map is of 'intra' of 'cis' type.
    resolution : str
        Resolution of required map. If contact map of input resolution is not found, ``None`` will be returned.
    workDir : str
        Name of directory where temporary files will be kept. These files will be automatically deleted.

    Returns
    -------
    object : None or :class:`gcMapExplorer.lib.ccmap.CCMAP`

    """

    # Check if h5py.File instance is provided or a name is provided
    fileOpened = False
    if isinstance(filename, str):
        hdf5 = h5py.File(filename)
        fileOpened = True
    elif isinstance(filename, h5py.File):
        hdf5 = filename
    else:
        raise TypeError ('{0} is not a gcmap file or h5py.File instance'.format(hdf5))

    # Working and output directory
    if workDir is None:
        workDir = config['Dirs']['WorkingDirectory']

    groupName = None
    xlabel = None
    ylabel = None

    if mapName is not None:
        if mapName not in hdf5:
            hdf5.close()
            return None
        groupName = mapName
        xlabel = hdf5[mapName].attrs['xlabel']
        ylabel = hdf5[mapName].attrs['ylabel']
    else:
        if chromAtX is not None:
            xlabel = chromAtX
            if chromAtY is not None:
                ylabel = chromAtY
            else:
                ylabel = xlabel
        elif chromAtY is not None:
            ylabel = chromAtY
            if chromAtX is not None:
                xlabel = chromAtX
            else:
                xlabel = ylabel
        else:
            raise AssertionError('Either mapName or chromAtX or chromAtY should be provided for further processing.')

        groupName = None
        if xlabel == ylabel:
            groupName = xlabel
        else:
            groupName = xlabel + '-' + ylabel

        if groupName not in hdf5:
            hdf5.close()
            return None

    mapType = None
    if xlabel == ylabel:
        mapType = 'intra'
    else:
        mapType = 'inter'

    if resolution is not None:
        if resolution not in hdf5[groupName].keys():
            hdf5.close()
            return None
    else:
        binsizes = []
        for key in hdf5[groupName].keys():
            if 'bNoData' not in key:
                binsizes.append( cmp.resolutionToBinsize(key) )
        binsizes = sorted(binsizes)
        resolution = cmp.binsizeToResolution(binsizes[0])
        del binsizes

    cmap = cmp.CCMAP(dtype=hdf5[groupName][resolution].dtype)

    for key in ['xlabel', 'ylabel']:
        cmap.__dict__[key] = hdf5[groupName].attrs[key]
    for key in ['minvalue', 'maxvalue', 'binsize']:
        cmap.__dict__[key] = hdf5[groupName][resolution].attrs[key]
    cmap.shape = (hdf5[groupName][resolution].attrs['xshape'], hdf5[groupName][resolution].attrs['yshape'])
    cmap.xticks = [0, cmap.shape[0]*cmap.binsize]
    cmap.yticks = [0, cmap.shape[1]*cmap.binsize]

    cmap.title = xlabel+'_vs_'+ylabel

    if resolution+'-bNoData' in hdf5[groupName].keys():
        cmap.bNoData = np.asarray( hdf5[groupName][resolution+'-bNoData'][:], dtype=np.bool )

    cmap.gen_matrix_file(workDir)
    cmap.make_writable()
    cmap.matrix[:] = hdf5[groupName][resolution][:]

    cmap.make_unreadable()

    if fileOpened:
        hdf5.close()

    return cmap


def addCCMap2GCMap(cmap, filename, compression='lzf', generateCoarse=True, coarsingMethod='sum', replaceCMap=True, logHandler=None):
    """ Add :class:`gcMapExplorer.lib.ccmap.CCMAP` to a gcmap file

    Parameters
    ----------
    cmap : :class:`gcMapExplorer.lib.ccmap.CCMAP`
        An instance of :class:`gcMapExplorer.lib.ccmap.CCMAP`, which will be added to gcmap file
    filename : str
        Name of ``gcmap`` file or h5py.File instance or GCMAP.hdf5 to which output data will be written.
    compression : str
        Compression method. Presently allowed : ``lzf`` for LZF compression and ``gzip`` for GZIP compression.
    generateCoarse : bool
        Also generates all coarser maps where resolutions will be coarsed by a factor of two, consequetively.
        e.g.: In case of 10 kb input resolution, downsampled maps of ``20kb``, ``40kb``, ``80kb``, ``160kb``, ``320kb`` etc.
        will be generated untill, map size is less than 500.
    coarsingMethod : str
        Method of downsampling. Three accepted methods are ``sum``: sum all values, ``mean``: Average of all values
        and ``max``: Maximum of all values.
    replaceCMap : bool
        Replace entire old ccmap data including resolutions and coarsed data.

    Returns
    -------
    success : bool
        If addition was successful ``True`` otherwise ``False``.

    """
    # logger
    logger = logging.getLogger('addCCMap2GCMap')
    if logHandler is not None:
        logger.propagate = False
        logger.addHandler(logHandler)
    logger.setLevel(logging.INFO)


    # Check compression method
    allowed_compressions = ['gzip', 'lzf']
    if compression not in allowed_compressions:
        raise KeyError('Compression method: "{0}" is not a valid keyword. Use "lzf" or "gzip".'.format(compression))

    # Check if h5py.File instance is provided or a name is provided
    fileOpened = False
    if isinstance(filename, str):
        hdf5 = h5py.File(filename)
        logger.info(' Opened file [{0}] for reading writing..'.format(filename))
        fileOpened = True
    elif isinstance(filename, h5py.File):
        hdf5 = filename
    else:
        raise TypeError ('{0} is not a gcmap file or h5py.File instance'.format(hdf5))

    cmap.make_readable()

    # Get x-label and y-label
    xlabel = cmap.xlabel
    ylabel = cmap.ylabel

    # Determine name of dataset/map on the basis of xlabel and ylabel
    groupName = None
    if xlabel == ylabel:
        groupName = xlabel
    else:
        groupName = xlabel+'-'+ylabel

    try:
        # If map is already present in file, remove it
        if replaceCMap and groupName in hdf5:
            logger.info(' Data for {0} is already present in [{1}], replacing it... '.format(groupName, filename))
            hdf5.pop(groupName)

        # Create group
        if groupName not in hdf5:
            group = hdf5.create_group(groupName)
        else:
            group = hdf5[groupName]

        group.attrs['xlabel'] = xlabel
        group.attrs['ylabel'] = ylabel
        group.attrs['compression'] = compression

        # Create and add map
        resolution = cmp.binsizeToResolution(cmap.binsize)
        if generateCoarse:
            logger.info(' Adding data to [{0}] for [{1}] ...'.format(filename, groupName))
        else:
            logger.info(' Adding data to [{0}] for [{1} - {2}] ...'.format(filename, groupName, resolution))

        # Remove old map
        if resolution in group:
            group.pop(resolution)


        # Add new map
        newCmap = group.create_dataset(resolution, cmap.shape, dtype=cmap.dtype, data=cmap.matrix, chunks=True, compression=compression, shuffle=True)

        # Save all other attributes
        if cmap.bNoData is not None:
            if resolution+'-bNoData' in group:
                group.pop(resolution+'-bNoData')
            group.create_dataset(resolution+'-bNoData', cmap.bNoData.shape, dtype=cmap.bNoData.dtype, data=cmap.bNoData, chunks=True, compression=compression, shuffle=True)

        # Get minimum value othar than zero
        if cmap.minvalue == 0:
            ma = np.ma.masked_equal(cmap.matrix, 0.0, copy=False)
            cmap.minvalue = ma.min()
            del ma

        newCmap.attrs['minvalue'] = cmap.minvalue
        newCmap.attrs['maxvalue'] = cmap.maxvalue
        newCmap.attrs['binsize'] = cmap.binsize
        newCmap.attrs['xshape'] = cmap.shape[0]
        newCmap.attrs['yshape'] = cmap.shape[1]

        logger.info('     ...Finished adding data for [{0}] ...'.format(groupName))
        cmap.make_unreadable()

        if generateCoarse:
            logger.info(' Generating downsampled maps for [{0}] ...'.format(groupName))
            gcmap = GCMAP(hdf5, mapName=groupName)
            gcmap.performDownSampling(method=coarsingMethod)
            del gcmap
            logger.info('     ... Finished downsampling for [{0}] ...'.format(groupName))

        if fileOpened:
            hdf5.close()
            logger.info(' Closed file [{0}]...'.format(filename))

        if logHandler is not None:
            logger.removeHandler( logHandler )

        return True

    except (KeyboardInterrupt, SystemExit) as e:
        # map might be incomplete, remove it
        if groupName in hdf5:
            hdf5.pop(groupName)

        hdf5.close()
        logger.info(' Closed file [{0}]...'.format(filename))

        cmap.make_unreadable()
        if logHandler is not None:
            logger.removeHandler( logHandler )

        raise e

    except:
        # Added map might be incomplete, remove it
        if groupName in hdf5:
            hdf5.pop(groupName)
        raise

        hdf5.close()
        logger.info(' Closed file [{0}]...'.format(filename))

        cmap.make_unreadable()
        if logHandler is not None:
            logger.removeHandler( logHandler )

        return False

def changeGCMapCompression(infile, outfile, compression, logHandler=None):
    """Change compression method in GCMAP file

    Change compression method in GCMAP file. Currently LZF and GZIP  compression is allowed. LZF is fast and moderately compressing algorithm.
    However, GZIP is slower with large compressing ratio.

    .. warning:: GCMAP with ``gzip`` compression can be universally read from any programming language using HDF5 library, however ``LZF`` compression can be only decompressed using Python h5py package.


    Parameters
    ----------
    infile : str
        Input GCMAP file
    outfile : str
        Output GCMAP file
    compression : str
        Method of compression: `lzf`` or ``gzip``


    """

    # logger
    logger = logging.getLogger('changeGCMapCompression')
    if logHandler is not None:
        logger.propagate = False
        logger.addHandler(logHandler)
    logger.setLevel(logging.INFO)

    # Check compression method
    allowed_compressions = ['gzip', 'lzf']
    if compression not in allowed_compressions:
        raise KeyError('Compression method: "{0}" is not a valid keyword. Use "lzf" or "gzip".'.format(compression))

    inHdf5 = h5py.File(infile, 'r')
    logger.info(' Opened file [{0}] for reading ..'.format(infile))

    outHdf5 = h5py.File(outfile)
    logger.info(' Opened file [{0}] for reading writing ..'.format(outfile))


    for mapName in inHdf5:
        try:
            if mapName in outHdf5:
                logger.info('Data for {0} is already present in [{1}], replacing it... '.format(mapName, outfile))
                outHdf5.pop(mapName)

            if inHdf5[mapName].attrs['compression'] == compression:
                logger.info('Group [{0}] is already compressed by requested {1} method. Skipping!!!' .format(mapName))
                continue

            outHdf5.create_group(mapName)
            for attr in inHdf5[mapName].attrs:
                outHdf5[mapName].attrs[attr] = inHdf5[mapName].attrs[attr]

            outHdf5[mapName].attrs['compression'] = compression

            for data in inHdf5[mapName].keys():
                group = outHdf5[mapName]
                dset = inHdf5[mapName][data]
                if compression == 'lzf':
                    newDset = group.create_dataset(data, dset.shape, dtype=dset.dtype, data=dset[:], chunks=True, compression="lzf", shuffle=True)
                else:
                    newDset = group.create_dataset(data, dset.shape, dtype=dset.dtype, data=dset[:], chunks=True, compression="gzip", shuffle=True, compression_opts=4)

                for attr in dset.attrs:
                    newDset.attrs[attr] = dset.attrs[attr]

            logger.info('     ...Finished compressing data for [{0}] ...'.format(mapName))

        except:
            # map might be incomplete, remove it
            if mapName in outHdf5:
                outHdf5.pop(mapName)

            outHdf5.close()
            inHdf5.close()
            logger.info(' Closed files [{0}] and [{1}]...'.format(infile, outfile))

            if logHandler is not None:
                logger.removeHandler( logHandler )

            raise


    inHdf5.close()
    outHdf5.close()
    logger.info(' Closed files [{0}] and [{1}]...'.format(infile, outfile))
    if logHandler is not None:
        logger.removeHandler( logHandler )

    return True



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import os, sys, re
import tempfile
import shlex, subprocess


from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.uic import loadUiType

from gcMapExplorer import importer
from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

from . import guiHelpers

# Determine absolute path to UIs directory. Relative path from this directory does not work.
DirToThisScript = os.path.dirname(os.path.abspath(__file__))
PathToUIs = os.path.join(DirToThisScript, 'UIs')

def main():
    app = QApplication(sys.argv)
    importer_interface = ImporterWindow()
    importer_interface.show()
    app.exec_()
    app.exit()

# Main Window Of Importer
pathToThisUI = os.path.join(PathToUIs, 'normalizer.ui')
Ui_ImporterWindow, ImporterWindowBase = loadUiType(pathToThisUI)
class ImporterWindow(ImporterWindowBase, Ui_ImporterWindow):
    def __init__(self):
        super(ImporterWindow, self).__init__()
        self.setupUi(self)

        # Hide tabbars from tab widget of specific options
        tabbars = self.specOptsTabWidget.findChildren(QTabBar)
        for tabbar in tabbars:
            tabbar.hide()

        # Resize hight and reduce size of log text box
        self.resize(self.width(), 680)
        self.splitter.setSizes([520, 160])

        # Remove maximize window buttons
        self.setWindowFlags( (self.windowFlags() | Qt.CustomizeWindowHint) & ~Qt.WindowMaximizeButtonHint)

        self.temporaryFiles = []
        self.command = None
        self.process = None

        self.setDefaultScratchDirs()
        self.connectButtons()
        self.connectLineEdits()

    def closeEvent(self, event):

        if self.process is not None:
            if self.process.state() == QProcess.Running:
                msg = " A process is still Running... \n" \
                        + "Are you sure to close?"
                msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg, QMessageBox.Yes | QMessageBox.No, self)
                msgBox.exec_()

                if msgBox.result() == QMessageBox.No:
                    close = False
                    event.ignore()
                else:
                    self.terminateProcessing()

                msgBox.close()

        if self.temporaryFiles:
            for i in range(len(self.temporaryFiles)):
                if os.path.isfile( self.temporaryFiles[i] ):
                    os.remove( self.temporaryFiles[i] )

    def setDefaultScratchDirs(self):
        defaultDir = config['Dirs']['WorkingDirectory']
        self.genOptsScratchDirLineEdit.setText(defaultDir)

    def connectButtons(self):
        self.methodCBox.currentIndexChanged.connect( self.specOptsTabWidget.setCurrentIndex )
        self.whatsThisButton.clicked.connect( QWhatsThis.enterWhatsThisMode )

        self.inputFileButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.inputFileButton.clicked.connect( self.openInputFile )

        self.outputFileButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.outputFileButton.clicked.connect( self.browseOutputFile )

        self.genOptsScratchDirButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.genOptsScratchDirButton.clicked.connect( self.browseScratchDir)

        # Set icon on start and stop button
        self.executeStartButton.setIcon( self.style().standardIcon(QStyle.SP_MediaPlay) )
        self.executeStopButton.setIcon( self.style().standardIcon(QStyle.SP_MediaStop) )

        # Connect start and stop button
        self.executeStartButton.clicked.connect( self.runCommand )
        self.executeStopButton.clicked.connect( self.terminateProcessing )

        # Connect clear button
        self.logOutputClearButton.clicked.connect( self.logOutputPlainTextEdit.clear )

    def connectLineEdits(self):
        # Filters Line Edit - Check only double is accepted
        self.genOptsPtndLineEdit.setValidator(QDoubleValidator())
        self.genOptsTdoLineEdit.setValidator(QDoubleValidator())

        # KR tolerance and MSCM -- only numbers
        self.specOptsKrTolLineEdit.setValidator(QDoubleValidator())
        self.specOptsKrMscmLineEdit.setValidator(QIntValidator())

        # IC tolerance and Iteration -- only numbers
        self.specOptsIcTolLineEdit.setValidator(QDoubleValidator())
        self.specOptsIcIterLineEdit.setValidator(QIntValidator())

        # vmin and vmax
        self.vminLineEdit.setValidator(QDoubleValidator())
        self.vmaxLineEdit.setValidator(QDoubleValidator())

        # Check for input file - file exist; determine file format
        self.inputFileLineEdit.editingFinished.connect( lambda: guiHelpers.checkFileExist(self.inputFileLineEdit, self) )
        self.inputFileLineEdit.editingFinished.connect( lambda: self.setFileFormat(self.inputFileLineEdit, self.inputFormatCBox) )

        # Check for output file determine file format
        self.outFileLineEdit.editingFinished.connect( lambda: self.setFileFormat(self.outFileLineEdit, self.outFormatCBox) )

        # Filters line edit - limits the value that should be entered in these line edits
        self.genOptsPtndLineEdit.editingFinished.connect( lambda: guiHelpers.constrainValueInLineEdit(self, self.genOptsPtndLineEdit, 0, 100,))
        self.genOptsTdoLineEdit.editingFinished.connect( lambda: guiHelpers.constrainValueInLineEdit(self, self.genOptsTdoLineEdit, 0, 1,))

        # Check if scratch directory exist when user try to write own directory
        self.genOptsScratchDirLineEdit.editingFinished.connect( lambda: guiHelpers.checkDirExist(self.genOptsScratchDirLineEdit, self) )


    def openInputFile(self):
        """To get compressed file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " map files (*.ccmap *.gcmap *.hicmap);; All files (*.*)"
        path = QFileDialog.getOpenFileName(self, 'Open File', '', file_choices)
        if path[0]:
            self.inputFileLineEdit.setText(path[0])
            self.setFileFormat(self.inputFileLineEdit, self.inputFormatCBox)   # Set input file format

    def setFileFormat(self, lineEdit, comboBox):
        """ Set file format in input and output combo box

        It try to get file extension from input file name and set this
        format to combo box.

        """
        filename = lineEdit.text()
        if filename:
            ext = guiHelpers.getFileExtension(filename)
            if ext == '.ccmap' or ext == '.hicmap':
                comboBox.setCurrentIndex(1)
            elif ext == '.gcmap':
                comboBox.setCurrentIndex(2)
            else:
                comboBox.setCurrentIndex(0)
        else:
            comboBox.setCurrentIndex(0) # Set input file format to None

    def browseScratchDir(self):
        """Browse and choose scratch directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Scratch Directory')
        if path:
            self.genOptsScratchDirLineEdit.setText(path)

    def browseOutputFile(self):
        """To open gcmap or ccmap file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " gcmap or ccmap file (*.gcmap *.ccmap);;All files(*.*)"
        path = QFileDialog.getSaveFileName(self, 'Select or Create File', '', file_choices, options=QFileDialog.DontConfirmOverwrite)
        if path[0]:
            self.outFileLineEdit.setText(path[0])
            self.setFileFormat(self.outFileLineEdit, self.outFormatCBox)   # Set input file format

    def isInputOutputFormatValid(self):
        """ Check Input and Output formats selected by user.
        """
        valid = True
        if self.inputFormatCBox.currentText() == 'gcmap' and self.outFormatCBox.currentText() == 'ccmap':
            msg = 'When input format is gcmap, output format should be only gcmap.'
            guiHelpers.showWarningMessageBox(msg, self)
            self.outFormatCBox.setFocus()
            valid = False

        if self.inputFormatCBox.currentIndex() == 0:
            msg = 'Input file format not known!!'
            guiHelpers.showWarningMessageBox(msg, self)
            self.inputFormatCBox.setFocus()
            valid = False

        if self.outFormatCBox.currentIndex() == 0:
            msg = 'Output file format not known!!'
            guiHelpers.showWarningMessageBox(msg, self)
            self.outFormatCBox.setFocus()
            valid = False

        return valid

    def isPTNDandTDOvalid(self):
        """ Check if both filters are applied
        """
        ptndText = self.genOptsPtndLineEdit.text()
        tdoText = self.genOptsTdoLineEdit.text()

        if ptndText and tdoText:
            msg = 'Both Percentile and Fraction filters cannot be used simultaneously!!'
            guiHelpers.showWarningMessageBox(msg, self)
            self.genOptsPtndLineEdit.setFocus()
            return False

        return True

    def isKrToleranceValid(self):
        """ Check if KR tolreance is given properly.

        * if tolerance < 1e-12 : Value is too small
        * if no tolerance : Need it

        """
        value = self.specOptsKrTolLineEdit.text()

        valid = True
        if value:
            value = float(value)
            if value < 1e-12:
                msg = 'Tolerance value too small!!\nThis precision is difficult to achieve.'
                guiHelpers.showWarningMessageBox(msg, self)
                self.specOptsKrTolLineEdit.selectAll()
                self.specOptsKrTolLineEdit.setFocus()
                valid = False
        else:
            msg = 'Provide the Tolerance value.'
            guiHelpers.showWarningMessageBox(msg, self)
            self.specOptsKrTolLineEdit.setFocus()
            valid = False

        return valid

    def isKrMscmValueValid(self):
        """ Check Map Size Celing value for memory is given
        It is required when input file is in gcmap format otherwise ignore it.
        """
        value = self.specOptsKrMscmLineEdit.text()

        valid = True
        if self.inputFormatCBox.currentText() == 'gcmap' and not value:
            msg = 'Provide Map Size Celing value for memory!'
            guiHelpers.showWarningMessageBox(msg, self)
            self.specOptsKrMscmLineEdit.setFocus()
            valid = False

        return valid

    def isIcTolAndIterationValid(self):
        Tol = self.specOptsIcTolLineEdit.text()
        Iteration = self.specOptsIcIterLineEdit.text()

        valid = True
        if Tol:
            value = float(Tol)
            if value < 1e-12:
                msg = 'Tolerance value too small!!\nThis precision is difficult to achieve.'
                guiHelpers.showWarningMessageBox(msg, self)
                self.specOptsIcTolLineEdit.setFocus()
                valid = False

        if not Tol and not Iteration:
            msg = 'Neither Tolerance nor Iteration value found !!! \n'
            guiHelpers.showWarningMessageBox(msg, self)
            self.specOptsIcTolLineEdit.setFocus()
            valid = False

        if not Iteration:
            msg = 'Iteration value not found.\nSetting it to default value.'
            guiHelpers.showWarningMessageBox(msg, self)
            self.specOptsIcIterLineEdit.setText('500')

        if not Tol:
            msg = 'Tolerance value not found.\nSetting it to default value.'
            guiHelpers.showWarningMessageBox(msg, self)
            self.specOptsIcTolLineEdit.setText('1e-4')

        return valid

    def readAndConstructCommand(self):
        """ Read all options from interface and construct the command
        """

        # Name of command
        programs = ['normKR', 'normIC', 'normMCFS']
        program = programs[self.methodCBox.currentIndex()]

        cmdDict = dict()

        # Input File
        inputFile = self.inputFileLineEdit.text()
        if not inputFile:
            msg = 'No input file!!'
            guiHelpers.showWarningMessageBox(msg, self)
            self.inputFileLineEdit.setFocus()
            return False
        cmdDict['-i'] = inputFile

        # Output File
        outputFile = self.outFileLineEdit.text()
        if not outputFile:
            msg = 'No output file!!'
            guiHelpers.showWarningMessageBox(msg, self)
            self.outFileLineEdit.setFocus()
            return False
        cmdDict['-o'] = outputFile

        # Input and output format
        inputFormat = None
        outputFormat = None
        if self.isInputOutputFormatValid():
            inputFormat = self.inputFormatCBox.currentText()
            outputFormat = self.outFormatCBox.currentText()
        else:
            return False
        cmdDict['-fi'] = inputFormat
        cmdDict['-fo'] = outputFormat

        # Vmin and vmax
        if self.vminLineEdit.text():
            cmdDict['-vmin'] = self.vminLineEdit.text()
        if self.vmaxLineEdit.text():
            cmdDict['-vmax'] = self.vmaxLineEdit.text()

        # Working or scratch directory
        workDir = self.genOptsScratchDirLineEdit.text()
        if not workDir:
            msg = 'No output file!!'
            guiHelpers.showWarningMessageBox(msg, self)
            self.genOptsScratchDirLineEdit.setFocus()
            return False
        cmdDict['-wd'] = '"{0}"'.format(workDir)

        # Compression method for gcmap output file
        if self.outFormatCBox.currentText() == 'gcmap':
            cmdDict['-cmeth'] = str(self.outFileCompressionCBox.currentText()).lower()

        # Filters
        if self.isPTNDandTDOvalid():
            if self.genOptsPtndLineEdit.text():
                cmdDict['-ptnd'] = float(self.genOptsPtndLineEdit.text())

            if self.genOptsTdoLineEdit.text():
                cmdDict['-tdo'] = float(self.genOptsTdoLineEdit.text())
        else:
            return False

        # Options related to KR normalization
        if program == 'normKR':

            # Tolerance
            krTol = None
            if self.isKrToleranceValid():
                krTol = float(self.specOptsKrTolLineEdit.text())
            else:
                return False
            cmdDict['-t'] = krTol

            # map size ceiling for memory - only need when input file is gcmap
            krMscm = None
            if self.isKrMscmValueValid():
                if self.specOptsKrMscmLineEdit.text():
                    krMscm = int( self.specOptsKrMscmLineEdit.text() )
                    cmdDict['-mscm'] = krMscm
            else:
                return False

            # Memory - only need when input file is ccmap
            if self.specOptsKrMemCBox.currentIndex() == 0:
                krMemory = 'RAM'
            else:
                krMemory = 'HDD'
            cmdDict['-m'] = krMemory

            # Construct the command
            self.constructKrCommand(cmdDict)

        # Options related to Iterative Correction
        if program == 'normIC':

            icTol = None
            icIter = None
            if self.isIcTolAndIterationValid():
                icTol = float( self.specOptsIcTolLineEdit.text() )
                icIter = int( self.specOptsIcIterLineEdit.text() )
            else:
                return False

            cmdDict['-t'] = icTol
            cmdDict['-c'] = icIter

            # Construct the command
            self.constructIcCommand(cmdDict)

        # Options related to MCFS
        if program == 'normMCFS':
            cmdDict['-s'] = str( self.specOptsMcfsStatsCBox.currentText() ).lower()

            # Construct the command
            self.constructMcfsCommand(cmdDict)


    def constructKrCommand(self, cmdDict):
        """ Construc normKI command
        """
        command = ' normKR '
        command += ' -i ' + cmdDict['-i']
        command += ' -fi ' + cmdDict['-fi']
        command += ' -o ' + cmdDict['-o']
        command += ' -fo ' + cmdDict['-fo']
        command += ' -t ' + str(cmdDict['-t'])
        command += ' -m ' + cmdDict['-m']

        if '-vmin' in cmdDict:
            command += ' -vmin ' + str(cmdDict['-vmin'])
        if '-vmax' in cmdDict:
            command += ' -vmax ' + str(cmdDict['-vmax'])
        if '-mscm' in cmdDict:
            command += ' -mscm ' + str(cmdDict['-mscm'])
        if '-cmeth' in cmdDict:
            command += ' -cmeth ' + cmdDict['-cmeth']
        command += ' -wd ' + cmdDict['-wd']
        if '-ptnd' in cmdDict:
            command += ' -ptnd ' + str(cmdDict['-ptnd'])
        if '-tdo' in cmdDict:
            command += ' -tdo ' + str(cmdDict['-tdo'])

        self.command = command

    def constructIcCommand(self, cmdDict):
        """ Construct normIC command
        """
        command = ' normIC '
        command += ' -i ' + cmdDict['-i']
        command += ' -fi ' + cmdDict['-fi']
        command += ' -o ' + cmdDict['-o']
        command += ' -fo ' + cmdDict['-fo']
        command += ' -t ' + str(cmdDict['-t'])
        command += ' -c ' + str(cmdDict['-c'])
        if '-vmin' in cmdDict:
            command += ' -vmin ' + str(cmdDict['-vmin'])
        if '-vmax' in cmdDict:
            command += ' -vmax ' + str(cmdDict['-vmax'])
        if '-cmeth' in cmdDict:
            command += ' -cmeth ' + cmdDict['-cmeth']
        command += ' -wd ' + cmdDict['-wd']
        if '-ptnd' in cmdDict:
            command += ' -ptnd ' + str(cmdDict['-ptnd'])
        if '-tdo' in cmdDict:
            command += ' -tdo ' + str(cmdDict['-tdo'])

        self.command = command

    def constructMcfsCommand(self, cmdDict):
        """ Construct normMCFS command
        """
        command = ' normMCFS '
        command += ' -i ' + cmdDict['-i']
        command += ' -fi ' + cmdDict['-fi']
        command += ' -o ' + cmdDict['-o']
        command += ' -fo ' + cmdDict['-fo']
        command += ' -s ' + str(cmdDict['-s'])
        if '-vmin' in cmdDict:
            command += ' -vmin ' + str(cmdDict['-vmin'])
        if '-vmax' in cmdDict:
            command += ' -vmax ' + str(cmdDict['-vmax'])
        if '-cmeth' in cmdDict:
            command += ' -cmeth ' + cmdDict['-cmeth']
        command += ' -wd ' + cmdDict['-wd']
        if '-ptnd' in cmdDict:
            command += ' -ptnd ' + str(cmdDict['-ptnd'])
        if '-tdo' in cmdDict:
            command += ' -tdo ' + str(cmdDict['-tdo'])

        self.command = command

    def runCommand(self):
        self.command = None
        self.readAndConstructCommand()
        if self.command is None:  return
        self.startProcess(self.command, self.executeStartButton)

    def startProcess(self, command, button):
        self.process = QProcess(self)
        self.process.start('gcMapExplorer', shlex.split(command))
        self.process.setProcessChannelMode( QProcess.MergedChannels )
        #self.process.setReadChannel( QProcess.StandardOutput )
        self.process.waitForStarted()

        self.process.readyReadStandardOutput.connect( self.writeLogOutputFromProcess )
        self.process.readyReadStandardError.connect( self.writeLogOutputFromProcess )
        self.process.finished.connect( lambda: self.finishedProcessing( button ) )

        self.methodCBox.setEnabled(False)
        button.setEnabled(False)
        self.logOutputPlainTextEdit.clear()

    def writeLogOutputFromProcess(self):
        out = bytes(self.process.readAllStandardOutput()).decode("utf-8")
        if out:
            out = out.rstrip()
            self.logOutputPlainTextEdit.appendPlainText( out  )
        out = bytes(self.process.readAllStandardError()).decode("utf-8")
        if out:
            out = out.rstrip()
            self.logOutputPlainTextEdit.appendPlainText( out  )

    def terminateProcessing(self):
        if self.process is None: return
        self.process.terminate()

    def finishedProcessing(self, button):
        button.setEnabled(True)
        self.methodCBox.setEnabled(True)
        self.process = None



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import os, re, sys
import tempfile
import configparser
from appdirs import system
from appdirs import AppDirs


defaultConfigText="""
[Dirs]
WorkingDirectory = {0}

[Programs]
bigWigInfo = None
bigWigToWig = None

""".format(tempfile.gettempdir())

configAppDirs = AppDirs('gcMapExplorer')

if system == 'win32':
    configFileName = 'gcMapExplorer.ini'
else:
    configFileName = 'gcMapExplorer.conf'

configFile = os.path.join(configAppDirs.user_config_dir, configFileName)


def defaultConfiguration():
    config = configparser.ConfigParser()
    config.read_string(defaultConfigText)
    return config

def updateConfig(section, option, value):
    if section not in config:
        return
    if option not in config[section]:
        return

    config[section][option] = value

    with open(configFile, 'w') as fout:
        config.write(fout)

def getConfig():
    if not os.path.exists(configFile):

        if not os.path.exists(configAppDirs.user_config_dir):
            try:
                os.makedirs(configAppDirs.user_config_dir)
            except OSError:
                pass


        config = defaultConfiguration()
        with open(configFile, 'w') as fout:
            config.write(fout)

    else:
        config = configparser.ConfigParser()
        config.read(configFile)

    return config


def initConfig():
    print('Configuration file not found...')
    print('Starting configuration:')

    while True:
        s = input('--> Write working directory name [current: {0}]'.format(config['Dirs']['WorkingDirectory']))
        if s:
            if not os.path.exists(s):
                print('Path {0} does not exist... Please retry...'.format(s))
            else:
                config['Dirs']['WorkingDirectory'] = s
                break

    with open(configFileName, 'w') as fout:
        config.write(fout)

    return config



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import os, sys, re
import tempfile
import shlex, subprocess


from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.uic import loadUiType

from gcMapExplorer import importer
from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

from . import guiHelpers

# Determine absolute path to UIs directory. Relative path from this directory does not work.
DirToThisScript = os.path.dirname(os.path.abspath(__file__))
PathToUIs = os.path.join(DirToThisScript, 'UIs')

def main():
    app = QApplication(sys.argv)
    importer_interface = ImporterWindow()
    importer_interface.show()
    app.exec_()
    app.exit()

class cooMatFormatTabWidgetbHelper:
    """ Helper class containing all member-functions related to COO matrix format tab-widget
    """

    def initCooMatFormatTabWidget(self):
        self.cooMatCommand = None

        self.cooMatIOTable.setColumnWidth(0, 350)
        self.cooMatIOTable.setColumnWidth(1, 100)
        self.cooMatIOTable.setColumnWidth(2, 100)

        self.cooMatArchivFileBrowsButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.cooMatIOAddFileButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.cooMatOutDirBrowseButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.cooMatScratchDirBrowsButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.cooMatGCMapOutSelectButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )

        self.cooMatTabRunButton.setIcon( self.style().standardIcon(QStyle.SP_MediaPlay) )
        self.cooMatTabStopButton.setIcon( self.style().standardIcon(QStyle.SP_MediaStop) )

        self.connectCooMatrixTabWidgets()

    def connectCooMatrixTabWidgets(self):
        self.cooMatIOTable.cellChanged.connect( self.cooMatInputFileCellChanged )
        self.cooMatIOAddFromFileButton.clicked.connect( self.cooMatAddInputFilesFromMetaFile )
        self.cooMatIOAddFileButton.clicked.connect( self.cooMatAddFileToSelectedRowCell )
        self.cooMatIOTableAddRowButton.clicked.connect( self.cooMatAddRowToTable )
        self.cooMatIOTableRemovRowButton.clicked.connect( self.cooMatRemoveRowFromTable )
        self.cooMatArchivFileBrowsButton.clicked.connect( self.cooMatBrowseCompressedFile )
        self.cooMatOutDirBrowseButton.clicked.connect( self.cooMatBrowseOutputDir )
        self.cooMatGCMapOutSelectButton.clicked.connect( self.cooMatOpenGCMapFile )
        self.cooMatScratchDirBrowsButton.clicked.connect( self.cooMatBrowseScratchDir )
        self.cooMatTabRunButton.clicked.connect( self.runCooMatrixCommand )
        self.cooMatTabStopButton.clicked.connect( self.terminateProcessing )

        self.cooMatArchivFileLineEdit.editingFinished.connect( lambda: checkFileExist(self.cooMatArchivFileLineEdit, self) )
        self.cooMatScratchDirLineEdit.editingFinished.connect( lambda: checkDirExist(self.cooMatScratchDirLineEdit, self) )
        self.cooMatOutDIrLineEdit.editingFinished.connect( lambda: checkDirExist(self.cooMatOutDIrLineEdit, self) )

    def cooMatSetLabelsToCells(self, row, xlabel=None, ylabel=None):
        """Convineint function to show xlabel and ylabel to table
        It is called several times below.
        """

        # Try to determine xlabel and ylabel from file name
        if xlabel is None:
            basename = os.path.basename( str( self.cooMatIOTable.item(row, 0).text() ) )
            g = re.match("Chr|chr", basename)
            if g:
                xlabel = re.split('_|-', basename[g.span()[0]:])[0]
                ylabel = xlabel
            else:
                xlabel = basename
                ylabel = xlabel

        if self.cooMatIOTable.item(row, 1) is None:
            self.cooMatIOTable.setItem( row, 1, QTableWidgetItem(0) )
        if self.cooMatIOTable.item(row, 2) is None:
            self.cooMatIOTable.setItem( row, 2, QTableWidgetItem(0) )

        self.cooMatIOTable.item(row, 1).setText(xlabel)
        self.cooMatIOTable.item(row, 2).setText(ylabel)

    def cooMatInputFileCellChanged(self, row, col):
        """When user write a input file name, automatically try to choose Labels
        """
        # Nothing to do when enterd on second column
        if col != 0:    return
        self.cooMatSetLabelsToCells(row)

    def cooMatAddInputFilesFromMetaFile(self):
        """To built input file list and labels from a meta input file
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " Text file (*.txt *.*)"
        path = QFileDialog.getOpenFileName(self, 'Load File', '', file_choices)

        # Make list of input files
        inputFiles, xlabels, ylabels = [], [], []
        if path[0]:
            fin = open(path[0], 'r')

            # Reading meta input file
            for line in fin:
                line = line.rstrip().lstrip()
                if not line.strip():    continue
                temp = re.split('\s+', line)

                # Check if meta input file has only one column
                if len(temp) < 2:
                    msg = "{0} contains only one column [{1}].\n \
                        Need at least two column with contact map file \
                        and xlabel !!!\n".format(path[0], temp[0])
                    msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg, QMessageBox.Ok, self)
                    msgBox.exec_()
                    msgBox.close()

                # Check is meta input file contains a file path
                if not self.cooMatArchivFileLineEdit.text():
                    if not os.path.exists( temp[0] ):
                        msg = 'File "{0}" not found. \n\nIf it is inside the compressed file, first select it.'.format(temp[0])
                        msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg, QMessageBox.Ok, self)
                        msgBox.exec_()
                        msgBox.close()
                        return

                inputFiles.append(temp[0])
                xlabels.append(temp[1])

                # Determine whether ylabel is present
                if len(temp) >= 3:
                    ylabels.append(temp[2])
                else:
                    ylabels.append(temp[1])

        # If there is list, add this list to table
        if inputFiles:
            for fidx in range(len(inputFiles)):

                r = self.cooMatIOTable.rowCount()
                if fidx > r-1 :
                    self.cooMatIOTable.insertRow(fidx)

                if self.cooMatIOTable.item(fidx, 0) is None:
                    self.cooMatIOTable.setItem( fidx, 0, QTableWidgetItem(0) )

                self.cooMatIOTable.item(fidx, 0).setText(inputFiles[fidx])
                self.cooMatSetLabelsToCells(fidx, xlabels[fidx], ylabels[fidx])

    def cooMatAddFileToSelectedRowCell(self):
        """Add a input file to selected Input Files cell
        """

        row, col = getSelectedRowColumnFromTable(self.cooMatIOTable)

        # If no cell is selected in "Input Files row", raise a message
        if row is None or col != 0:
            msgBox = QMessageBox(self)
            msgBox.setWindowModality(Qt.WindowModal)
            msgBox.setWindowTitle('Information')
            msgBox.setIcon(QMessageBox.Information)
            msgBox.setText('         No Input Files cell selected. \n\nSelect a Input Files cell on table to add file.')
            msgBox.setStandardButtons(QMessageBox.Cancel)
            msgBox.exec_()
            msgBox.close()
            return

        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " Text file (*.txt *.*)"
        path = QFileDialog.getOpenFileName(self, 'Load File', '', file_choices)
        if path[0]:
            # Add input file to first column
            self.cooMatIOTable.item(row, 0).setText(path[0])
            self.cooMatSetLabelsToCells(row)

    def cooMatAddRowToTable(self):
        """Add a new row at the end of cooMatIOTable
        """
        self.cooMatIOTable.insertRow(self.cooMatIOTable.rowCount())

    def cooMatRemoveRowFromTable(self):
        """Remove a selected row from cooMatIOTable
        """
        # Total number of row
        row, col = getSelectedRowColumnFromTable(self.cooMatIOTable)

        # If no cell is selected in "Input Files row", raise a message
        if row is None:
            msgBox = QMessageBox(self)
            msgBox.setWindowModality(Qt.WindowModal)
            msgBox.setWindowTitle('Information')
            msgBox.setIcon(QMessageBox.Information)
            msgBox.setText('       A row is not selected \n\nSelect a Row to remove from table')
            msgBox.setStandardButtons(QMessageBox.Cancel)
            msgBox.exec_()
            msgBox.close()
            return

        self.cooMatIOTable.removeRow(row)

    def cooMatBrowseCompressedFile(self):
        """To get compressed file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " Archive file (*.zip *.tar.gz *.tar.bz2)"
        path = QFileDialog.getOpenFileName(self, 'Open File', '', file_choices)
        if path[0]:
            self.cooMatArchivFileLineEdit.setText(path[0])

    def cooMatBrowseOutputDir(self):
        """Browse and choose output directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Output Directory')
        if path:
            self.cooMatOutDIrLineEdit.setText(path)

    def cooMatBrowseScratchDir(self):
        """Browse and choose scratch directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Scratch Directory')
        if path:
            self.cooMatScratchDirLineEdit.setText(path)

    def cooMatOpenGCMapFile(self):
        """To open gcmap file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " gcmap file (*.gcmap);;All files(*.*)"
        path = QFileDialog.getSaveFileName(self, 'Select or Create File', '', file_choices, options=QFileDialog.DontConfirmOverwrite)
        if path[0]:
            self.cooMatGCMapOutLineEdit.setText(path[0])

    def readAndConstructCooMatCommand(self):
        """Read and Construct the command line
        """
        self.cooMatCommand = None
        options = dict()

        if self.cooMatArchivFileLineEdit.text():
            inputCompressedFile = str( self.cooMatArchivFileLineEdit.text() )
        else:
            inputCompressedFile = 'None'

        options['inputCompressedFile'] = inputCompressedFile

        inputFiles, xlabels, ylabels = [], [], []
        rows = self.cooMatIOTable.rowCount()
        for row in range(rows):
            if self.cooMatIOTable.item(row, 0) is not None and self.cooMatIOTable.item(row, 0).text() :
                inputFiles.append( str( self.cooMatIOTable.item(row, 0).text() ) )
                xlabels.append( str( self.cooMatIOTable.item(row, 1).text() ) )
                ylabels.append( str( self.cooMatIOTable.item(row, 2).text() ) )

        if inputCompressedFile == 'None' and not inputFiles:
            msg = "Neither input compressed file, nor Input Files are given!!!"
            showWarningMessageBox(msg, self)
            return False

        options['inputFiles'] = inputFiles
        options['xlabels'] = xlabels
        options['ylabels'] = ylabels

        options['mapType'] = str( self.cooMatMapTypeCBox.currentText() )
        if self.cooMatResolutionLineEdit.text():
            options['resolution'] = self.cooMatResolutionLineEdit.text()
        else:
            options['resolution'] = 'None'

        options['coordinate'] = str( self.cooMatCoordTypeCBox.currentText() )
        options['workDir'] = '"{0}"'.format(self.cooMatScratchDirLineEdit.text())

        if not self.cooMatCCMapGroupBox.isChecked() \
                        and not self.cooMatGCMapBoxGroupBox.isChecked():
            showWarningMessageBox("No ccmap or gcmap output !!!", self)
            return False

        options['ccmap'] = False
        if self.cooMatCCMapGroupBox.isChecked():
            ccmapSuffix = str( self.cooMatCCMapOutSuffixLineEdit.text() )
            if not ccmapSuffix:
                msg = "No suffix provided for ccmap files. \n" \
                        + "Please provide a suffix for proper name."
                showWarningMessageBox(msg, self)
                self.cooMatCCMapOutSuffixLineEdit.setFocus()
                return False

            outDir = str( self.cooMatOutDIrLineEdit.text() )
            if not outDir:
                msg = "No Output Directory provided for ccmap files \n" \
                        + "Please select a directory to save ccmap files."
                showWarningMessageBox(msg, self)
                self.cooMatOutDIrLineEdit.setFocus()
                return False

            options['ccmap'] = True
            options['ccmapSuffix'] = ccmapSuffix
            options['outDir'] = '"{0}"'.format(outDir)

        options['gcmap'] = False
        if self.cooMatGCMapBoxGroupBox.isChecked():
            fileGCMap = str( self.cooMatGCMapOutLineEdit.text() )
            if not fileGCMap:
                msg = "No Output gcmap file is created or selected \n" \
                        + "Please select or create a gcmap file."
                showWarningMessageBox(msg, self)
                self.cooMatGCMapOutLineEdit.setFocus()
                return False

            options['gcmap'] = True
            options['fileGCMap'] = '"{0}"'.format(fileGCMap)
            options['compression'] = str( self.cooMatGCMapCompressCBox.currentText() ).lower()
            options['coarsingMethod'] = str( self.cooMatGCMapDownsampleCBox.currentText() ).lower()

        self.cooMatrixConstructCommand(options)

    def cooMatrixConstructCommand(self, opts):
        """Construct the command line
        """

        command = ' coo2cmap '

        if opts['inputFiles']:
            of, inputMetaFileName = tempfile.mkstemp(
                                        dir=config['Dirs']['WorkingDirectory'],
                                        text=True)
            os.close(of)

            fout = open(inputMetaFileName, 'w')
            for i in range(len(opts['inputFiles'])):
                fout.write('{0}\t{1}\t{2}\n'.format(opts['inputFiles'][i],
                                                    opts['xlabels'][i],
                                                    opts['ylabels'][i]))
            fout.close()

            command += ' -i ' + inputMetaFileName
            self.temporaryFiles.append(inputMetaFileName)

        if opts['inputCompressedFile'] != 'None':
            command += ' -ic ' +  opts['inputCompressedFile']

        if opts['resolution'] != 'None':
            command += ' -r ' + opts['resolution']

        if opts['coordinate'] == 'index':
            command += ' -idx '

        if opts['ccmap']:
            command += ' -ccm ' + opts['ccmapSuffix']
            command += ' -od ' + opts['outDir']

        if opts['gcmap']:
            command += ' -gcm ' + opts['fileGCMap']
            command += ' -cmeth ' + opts['compression']
            command += ' -dmeth ' + opts['coarsingMethod']

        command += ' -mt ' + opts['mapType']
        command += ' -wd ' + opts['workDir']

        self.cooMatCommand = command

class homerFormatTabWidgetHelper:
    """ Helper class containing all member-functions related to HOMER matrix format tab-widget
    """

    def initHomerFormatTabWidget(self):
        self.homerCommand = None

        self.homerInputFileBrowsButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.homerOutDirBrowseButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.homerScratchDirBrowsButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.homerGCMapOutSelectButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )

        self.homerTabRunButton.setIcon( self.style().standardIcon(QStyle.SP_MediaPlay) )
        self.homerTabStopButton.setIcon( self.style().standardIcon(QStyle.SP_MediaStop) )

        self.connectHomerTabWidgets()

    def connectHomerTabWidgets(self):
        self.homerInputFileBrowsButton.clicked.connect( self.homerBrowseInputFile )
        self.homerScratchDirBrowsButton.clicked.connect( self.homerBrowseScratchDir )
        self.homerOutDirBrowseButton.clicked.connect( self.homerBrowseOutputDir )
        self.homerGCMapOutSelectButton.clicked.connect( self.homerOpenGCMapFile )

        self.homerTabRunButton.clicked.connect( self.runHomerCommand )
        self.homerTabStopButton.clicked.connect( self.terminateProcessing )

        self.homerInputFileLineEdit.editingFinished.connect( lambda: checkFileExist(self.homerInputFileLineEdit, self) )
        self.homerScratchDirLineEdit.editingFinished.connect( lambda: checkDirExist(self.homerScratchDirLineEdit, self) )
        self.homerOutDIrLineEdit.editingFinished.connect( lambda: checkDirExist(self.homerOutDIrLineEdit, self) )


    def homerBrowseInputFile(self):
        """To get compressed file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " Text file (*.txt *.dat);; All file (*.*)"
        path = QFileDialog.getOpenFileName(self, 'Open File', '', file_choices)
        if path[0]:
            self.homerInputFileLineEdit.setText(path[0])

    def homerBrowseOutputDir(self):
        """Browse and choose output directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Output Directory')
        if path:
            self.homerOutDIrLineEdit.setText(path)

    def homerBrowseScratchDir(self):
        """Browse and choose scratch directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Scratch Directory')
        if path:
            self.homerScratchDirLineEdit.setText(path)

    def homerOpenGCMapFile(self):
        """To open gcmap file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " gcmap file (*.gcmap);;All files(*.*)"
        path = QFileDialog.getSaveFileName(self, 'Select or Create File', '', file_choices, options=QFileDialog.DontConfirmOverwrite)
        if path[0]:
            self.homerGCMapOutLineEdit.setText(path[0])

    def readAndConstructHomerCommand(self):
        """Read and construct the command line
        """
        self.homerCommand = None
        options = dict()

        inputFile = None
        if self.homerInputFileLineEdit.text():
            inputFile = str( self.homerInputFileLineEdit.text() )
        else:
            self.homerInputFileLineEdit.setFocus()
            showWarningMessageBox("No input file given !!!", self)
            return False

        options['inputFile'] = inputFile

        options['workDir'] = '"{0}"'.format(self.homerScratchDirLineEdit.text())

        if not self.homerCCMapGroupBox.isChecked() \
                        and not self.homerGCMapGroupBox.isChecked():
            showWarningMessageBox("No ccmap or gcmap output !!!", self)
            return False

        options['ccmap'] = False
        if self.homerCCMapGroupBox.isChecked():
            ccmapSuffix = str( self.homerOutSuffixLineEdit.text() )
            if not ccmapSuffix:
                msg = "No suffix provided for ccmap files. \n" \
                        + "Please provide a suffix for proper name."
                showWarningMessageBox(msg, self)
                self.homerOutSuffixLineEdit.setFocus()
                return False

            outDir = str( self.homerOutDIrLineEdit.text() )
            if not outDir:
                msg = "No Output Directory provided for ccmap files \n" \
                        + "Please select a directory to save ccmap files."
                showWarningMessageBox(msg, self)
                self.homerOutDIrLineEdit.setFocus()
                return False

            options['ccmap'] = True
            options['ccmapSuffix'] = ccmapSuffix
            options['outDir'] = '"{0}"'.format(outDir)

        options['gcmap'] = False
        if self.homerGCMapGroupBox.isChecked():
            fileGCMap = str( self.homerGCMapOutLineEdit.text() )
            if not fileGCMap:
                msg = "No Output gcmap file is created or selected \n" \
                        + "Please select or create a gcmap file."
                showWarningMessageBox(msg, self)
                self.homerGCMapOutLineEdit.setFocus()
                return False

            options['gcmap'] = True
            options['fileGCMap'] = '"{0}"'.format(fileGCMap)
            options['compression'] = str( self.homerGCMapCompressCBox.currentText() ).lower()
            options['coarsingMethod'] = str( self.homerGCMapDownsampleCBox.currentText() ).lower()

        self.homerConstructCommand(options)

    def homerConstructCommand(self, opts):
        """Construct the command line
        """
        command = ' homer2cmap '

        command += ' -i ' +  opts['inputFile']

        if opts['ccmap']:
            command += ' -ccm ' + opts['ccmapSuffix']
            command += ' -od ' + opts['outDir']

        if opts['gcmap']:
            command += ' -gcm ' + opts['fileGCMap']
            command += ' -cmeth ' + opts['compression']
            command += ' -dmeth ' + opts['coarsingMethod']

        command += ' -wd ' + opts['workDir']

        self.homerCommand = command

class binContactFormatTabWidgetHelper:
    """ Helper class containing all member-functions related to Bin-Contact files pair tab-widget
    """

    def initBinContactFormatTabWidget(self):
        self.binContactCommand = None

        self.binContactInputBinFileBrowsButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.binContactInputContactFileBrowsButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.binContactOutDirBrowseButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.binContactScratchDirBrowsButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.binContactGCMapOutSelectButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )

        self.binContactTabRunButton.setIcon( self.style().standardIcon(QStyle.SP_MediaPlay) )
        self.binContactTabStopButton.setIcon( self.style().standardIcon(QStyle.SP_MediaStop) )

        self.connectBinContactTabWidgets()

    def connectBinContactTabWidgets(self):
        self.binContactInputBinFileBrowsButton.clicked.connect( self.binContactBrowseInputBinFile )
        self.binContactInputContactFileBrowsButton.clicked.connect( self.binContactBrowseInputContactFile )
        self.binContactScratchDirBrowsButton.clicked.connect( self.binContactBrowseScratchDir )
        self.binContactOutDirBrowseButton.clicked.connect( self.binContactBrowseOutputDir )
        self.binContactGCMapOutSelectButton.clicked.connect( self.binContactOpenGCMapFile )

        self.binContactTabRunButton.clicked.connect( self.runBinContactCommand )
        self.binContactTabStopButton.clicked.connect( self.terminateProcessing )

        self.binContactInputBinFileLineEdit.editingFinished.connect( lambda: checkFileExist(self.binContactInputBinFileLineEdit, self) )
        self.binContactInputContactFileLineEdit.editingFinished.connect( lambda: checkFileExist(self.binContactInputContactFileLineEdit, self) )
        self.binContactScratchDirLineEdit.editingFinished.connect( lambda: checkDirExist(self.binContactScratchDirLineEdit, self) )
        self.binContactOutDIrLineEdit.editingFinished.connect( lambda: checkDirExist(self.binContactOutDIrLineEdit, self) )


    def binContactBrowseInputBinFile(self):
        """To get compressed file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " Text file (*.txt *.dat);; All file (*.*)"
        path = QFileDialog.getOpenFileName(self, 'Open File', '', file_choices)
        if path[0]:
            self.binContactInputBinFileLineEdit.setText(path[0])

    def binContactBrowseInputContactFile(self):
        """To get compressed file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " Text file (*.txt *.dat);; All file (*.*)"
        path = QFileDialog.getOpenFileName(self, 'Open File', '', file_choices)
        if path[0]:
            self.binContactInputContactFileLineEdit.setText(path[0])

    def binContactBrowseOutputDir(self):
        """Browse and choose output directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Output Directory')
        if path:
            self.binContactOutDIrLineEdit.setText(path)

    def binContactBrowseScratchDir(self):
        """Browse and choose scratch directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Scratch Directory')
        if path:
            self.binContactScratchDirLineEdit.setText(path)

    def binContactOpenGCMapFile(self):
        """To open gcmap file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " gcmap file (*.gcmap);;All files(*.*)"
        path = QFileDialog.getSaveFileName(self, 'Select or Create File', '', file_choices, options=QFileDialog.DontConfirmOverwrite)
        if path[0]:
            self.binContactGCMapOutLineEdit.setText(path[0])

    def readAndConstructBinContactCommand(self):
        """Read and construct the command line
        """
        self.binContactCommand = None
        options = dict()

        inputBinFile = None
        if self.binContactInputBinFileLineEdit.text():
            inputFile = str( self.binContactInputBinFileLineEdit.text() )
        else:
            self.binContactInputBinFileLineEdit.setFocus()
            showWarningMessageBox("No input file given !!!", self)
            return False
        options['inputBinFile'] = inputFile

        inputContactFile = None
        if self.binContactInputContactFileLineEdit.text():
            inputFile = str( self.binContactInputContactFileLineEdit.text() )
        else:
            self.binContactInputContactFileLineEdit.setFocus()
            showWarningMessageBox("No input file given !!!", self)
            return False
        options['inputContactFile'] = inputFile

        options['workDir'] = '"{0}"'.format(self.binContactScratchDirLineEdit.text())

        if not self.binContactCCMapGroupBox.isChecked() \
                        and not self.binContactGCMapGroupBox.isChecked():
            showWarningMessageBox("No ccmap or gcmap output !!!", self)
            return False

        options['ccmap'] = False
        if self.binContactCCMapGroupBox.isChecked():
            ccmapSuffix = str( self.binContactOutSuffixLineEdit.text() )
            if not ccmapSuffix:
                msg = "No suffix provided for ccmap files. \n" \
                        + "Please provide a suffix for proper name."
                showWarningMessageBox(msg, self)
                self.binContactOutSuffixLineEdit.setFocus()
                return False

            outDir = str( self.binContactOutDIrLineEdit.text() )
            if not outDir:
                msg = "No Output Directory provided for ccmap files \n" \
                        + "Please select a directory to save ccmap files."
                showWarningMessageBox(msg, self)
                self.binContactOutDIrLineEdit.setFocus()
                return False

            options['ccmap'] = True
            options['ccmapSuffix'] = ccmapSuffix
            options['outDir'] = '"{0}"'.format(outDir)

        options['gcmap'] = False
        if self.binContactGCMapGroupBox.isChecked():
            fileGCMap = str( self.binContactGCMapOutLineEdit.text() )
            if not fileGCMap:
                msg = "No Output gcmap file is created or selected \n" \
                        + "Please select or create a gcmap file."
                showWarningMessageBox(msg, self)
                self.binContactGCMapOutLineEdit.setFocus()
                return False

            options['gcmap'] = True
            options['fileGCMap'] = '"{0}"'.format(fileGCMap)
            options['compression'] = str( self.binContactGCMapCompressCBox.currentText() ).lower()
            options['coarsingMethod'] = str( self.binContactGCMapDownsampleCBox.currentText() ).lower()

        self.binContactConstructCommand(options)

    def binContactConstructCommand(self, opts):
        """Construct the command line
        """
        command = ' binContact2cmap '

        command += ' -ib ' +  opts['inputBinFile']
        command += ' -ic ' +  opts['inputContactFile']

        if opts['ccmap']:
            command += ' -ccm ' + opts['ccmapSuffix']
            command += ' -od ' + opts['outDir']

        if opts['gcmap']:
            command += ' -gcm ' + opts['fileGCMap']
            command += ' -cmeth ' + opts['compression']
            command += ' -dmeth ' + opts['coarsingMethod']

        command += ' -wd ' + opts['workDir']

        self.binContactCommand = command

class pairCooMatFormatTabWidgetHelper:
    """ Helper class containing all member-functions related to paired-COO matrix file tab-widget
    """

    def initPairCooMatFormatTabWidget(self):
        self.pairCooMatCommand = None

        self.pairCooMatInputFileBrowsButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.pairCooMatOutDirBrowseButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.pairCooMatScratchDirBrowsButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.pairCooMatGCMapOutSelectButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )

        self.pairCooMatTabRunButton.setIcon( self.style().standardIcon(QStyle.SP_MediaPlay) )
        self.pairCooMatTabStopButton.setIcon( self.style().standardIcon(QStyle.SP_MediaStop) )

        self.connectPairCooMatTabWidgets()

    def connectPairCooMatTabWidgets(self):
        self.pairCooMatInputFileBrowsButton.clicked.connect( self.pairCooMatBrowseInputFile )
        self.pairCooMatScratchDirBrowsButton.clicked.connect( self.pairCooMatBrowseScratchDir )
        self.pairCooMatOutDirBrowseButton.clicked.connect( self.pairCooMatBrowseOutputDir )
        self.pairCooMatGCMapOutSelectButton.clicked.connect( self.pairCooMatOpenGCMapFile )

        self.pairCooMatTabRunButton.clicked.connect( self.runPairCooMatCommand )
        self.pairCooMatTabStopButton.clicked.connect( self.terminateProcessing )

        self.pairCooMatInputFileLineEdit.editingFinished.connect( lambda: checkFileExist(self.pairCooMatInputFileLineEdit, self) )
        self.pairCooMatScratchDirLineEdit.editingFinished.connect( lambda: checkDirExist(self.pairCooMatScratchDirLineEdit, self) )
        self.pairCooMatOutDIrLineEdit.editingFinished.connect( lambda: checkDirExist(self.pairCooMatOutDIrLineEdit, self) )


    def pairCooMatBrowseInputFile(self):
        """To get compressed file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " Text file (*.txt *.dat);; All file (*.*)"
        path = QFileDialog.getOpenFileName(self, 'Open File', '', file_choices)
        if path[0]:
            self.pairCooMatInputFileLineEdit.setText(path[0])

    def pairCooMatBrowseOutputDir(self):
        """Browse and choose output directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Output Directory')
        if path:
            self.pairCooMatOutDIrLineEdit.setText(path)

    def pairCooMatBrowseScratchDir(self):
        """Browse and choose scratch directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Scratch Directory')
        if path:
            self.pairCooMatScratchDirLineEdit.setText(path)

    def pairCooMatOpenGCMapFile(self):
        """To open gcmap file with full path
        """
        # A dialog box will be displayed to select a text file and path will be stored in the cell
        file_choices = " gcmap file (*.gcmap);;All files(*.*)"
        path = QFileDialog.getSaveFileName(self, 'Select or Create File', '', file_choices, options=QFileDialog.DontConfirmOverwrite)
        if path[0]:
            self.pairCooMatGCMapOutLineEdit.setText(path[0])

    def readAndConstructPairCooMatCommand(self):
        """Read and construct the command line
        """
        self.pairCooMatCommand = None
        options = dict()

        inputFile = None
        if self.pairCooMatInputFileLineEdit.text():
            inputFile = str( self.pairCooMatInputFileLineEdit.text() )
        else:
            self.pairCooMatInputFileLineEdit.setFocus()
            showWarningMessageBox("No input file given !!!", self)
            return False
        options['-i'] = inputFile

        options['-wd'] = '"{0}"'.format(self.pairCooMatScratchDirLineEdit.text())

        if not self.pairCooMatCCMapGroupBox.isChecked() \
                        and not self.pairCooMatGCMapGroupBox.isChecked():
            showWarningMessageBox("No ccmap or gcmap output !!!", self)
            return False

        ccmapSuffix = None
        if self.pairCooMatCCMapGroupBox.isChecked():
            ccmapSuffix = str( self.pairCooMatOutSuffixLineEdit.text() )
            if not ccmapSuffix:
                msg = "No suffix provided for ccmap files. \n" \
                        + "Please provide a suffix for proper name."
                showWarningMessageBox(msg, self)
                self.pairCooMatOutSuffixLineEdit.setFocus()
                return False

            outDir = str( self.pairCooMatOutDIrLineEdit.text() )
            if not outDir:
                msg = "No Output Directory provided for ccmap files \n" \
                        + "Please select a directory to save ccmap files."
                showWarningMessageBox(msg, self)
                self.pairCooMatOutDIrLineEdit.setFocus()
                return False

            options['-ccm'] = ccmapSuffix
            options['-od'] = '"{0}"'.format(outDir)

        if self.pairCooMatGCMapGroupBox.isChecked():
            fileGCMap = str( self.pairCooMatGCMapOutLineEdit.text() )
            if not fileGCMap:
                msg = "No Output gcmap file is created or selected \n" \
                        + "Please select or create a gcmap file."
                showWarningMessageBox(msg, self)
                self.pairCooMatGCMapOutLineEdit.setFocus()
                return False

            options['-gcm'] = '"{0}"'.format(fileGCMap)
            options['-cmeth'] = str( self.pairCooMatGCMapCompressCBox.currentText() ).lower()
            options['-dmeth'] = str( self.pairCooMatGCMapDownsampleCBox.currentText() ).lower()

        self.pairCooMatConstructCommand(options)

    def pairCooMatConstructCommand(self, opts):
        """Construct the command line
        """
        command = ' pairCoo2cmap '
        command += ' -i ' +  opts['-i']

        if '-ccm' in opts:
            command += ' -ccm ' + opts['-ccm']
            command += ' -od ' + opts['-od']

        if '-gcm' in opts:
            command += ' -gcm ' + opts['-gcm']
            command += ' -cmeth ' + opts['-cmeth']
            command += ' -dmeth ' + opts['-dmeth']

        command += ' -wd ' + opts['-wd']

        self.pairCooMatCommand = command

# Main Window Of Importer
pathToThisUI = os.path.join(PathToUIs, 'importer.ui')
Ui_ImporterWindow, ImporterWindowBase = loadUiType(pathToThisUI)
class ImporterWindow(ImporterWindowBase, Ui_ImporterWindow, cooMatFormatTabWidgetbHelper,
                    homerFormatTabWidgetHelper, binContactFormatTabWidgetHelper,
                    pairCooMatFormatTabWidgetHelper):
    def __init__(self):
        super(ImporterWindow, self).__init__()
        self.setupUi(self)

        # Hide tabbars
        tabbars = self.findChildren(QTabBar)
        for tabbar in tabbars:
            tabbar.hide()

        # Resize hight and reduce size of log text box
        self.resize(self.width(), 680)
        self.splitter.setSizes([500, 180])

        self.temporaryFiles = []
        self.process = None

        self.initCooMatFormatTabWidget()
        self.initHomerFormatTabWidget()
        self.initBinContactFormatTabWidget()
        self.initPairCooMatFormatTabWidget()

        self.setDefaultScratchDirs()
        self.connectMainButtons()

    def closeEvent(self, event):

        if self.process is not None:
            if self.process.state() == QProcess.Running:
                msg = " A process is still Running... \n" \
                        + "Are you sure to close?"
                msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg, QMessageBox.Yes | QMessageBox.No, self)
                msgBox.exec_()

                if msgBox.result() == QMessageBox.No:
                    close = False
                    event.ignore()
                else:
                    self.terminateProcessing()

                msgBox.close()

        if self.temporaryFiles:
            for i in range(len(self.temporaryFiles)):
                if os.path.isfile( self.temporaryFiles[i] ):
                    os.remove( self.temporaryFiles[i] )

    def connectMainButtons(self):
        self.inputSelectorQCBox.currentIndexChanged.connect( self.InputsTabWidget.setCurrentIndex )
        self.whatsThisButton.clicked.connect( QWhatsThis.enterWhatsThisMode )
        self.logOutputClearButton.clicked.connect( self.logOutputPlainTextEdit.clear )

    def setDefaultScratchDirs(self):
        defaultDir = config['Dirs']['WorkingDirectory']
        self.cooMatScratchDirLineEdit.setText(defaultDir)
        self.homerScratchDirLineEdit.setText(defaultDir)
        self.binContactScratchDirLineEdit.setText(defaultDir)
        self.pairCooMatScratchDirLineEdit.setText(defaultDir)

    def runCooMatrixCommand(self):
        self.readAndConstructCooMatCommand()
        if self.cooMatCommand is None:  return
        self.startProcess(self.cooMatCommand, self.cooMatTabRunButton)

    def runHomerCommand(self):
        self.readAndConstructHomerCommand()
        if self.homerCommand is None:  return
        self.startProcess(self.homerCommand, self.homerTabRunButton)

    def runBinContactCommand(self):
        self.readAndConstructBinContactCommand()
        if self.binContactCommand is None:  return
        self.startProcess(self.binContactCommand, self.binContactTabRunButton)

    def runPairCooMatCommand(self):
        self.readAndConstructPairCooMatCommand()
        if self.pairCooMatCommand is None:  return
        self.startProcess(self.pairCooMatCommand, self.pairCooMatTabRunButton)

    def startProcess(self, command, button):
        self.process = QProcess(self)
        self.process.start('gcMapExplorer', shlex.split(command))
        self.process.setProcessChannelMode( QProcess.MergedChannels )
        #self.process.setReadChannel( QProcess.StandardOutput )
        self.process.waitForStarted()

        self.process.readyReadStandardOutput.connect( self.writeLogOutputFromProcess )
        self.process.readyReadStandardError.connect( self.writeLogOutputFromProcess )
        self.process.finished.connect( lambda: self.finishedProcessing( button ) )

        self.inputSelectorQCBox.setEnabled(False)
        button.setEnabled(False)
        self.logOutputPlainTextEdit.clear()

    def writeLogOutputFromProcess(self):
        out = bytes(self.process.readAllStandardOutput()).decode("utf-8")
        if out:
            out = out.rstrip()
            self.logOutputPlainTextEdit.appendPlainText( out  )
        out = bytes(self.process.readAllStandardError()).decode("utf-8")
        if out:
            out = out.rstrip()
            self.logOutputPlainTextEdit.appendPlainText( out  )

    def terminateProcessing(self):
        if self.process is None: return
        self.process.terminate()

    def finishedProcessing(self, button):
        button.setEnabled(True)
        self.inputSelectorQCBox.setEnabled(True)
        self.process = None


def getSelectedRowColumnFromTable(table):
    # Total number of row
    r = table.rowCount()
    c = table.columnCount()

    # Determine which cell of row is selected
    selectedRow = None
    selectedCol = None
    for i in range(r):
        for j in range(c):
            if table.item(i, j) is None:
                table.setItem( i, j, QTableWidgetItem(0) )

            if table.item(i, j).isSelected():
                selectedRow = i
                selectedCol = j
                break

    return selectedRow, selectedCol


def checkFileExist(lineEdit, qwidget):
    if not lineEdit.text(): return
    filename = str( lineEdit.text() )
    if not os.path.isfile( filename ):
        msg = "[ {0} ] \n Not found !!!".format(filename)
        msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg,
                                                    QMessageBox.Ok, qwidget)
        msgBox.exec_()
        msgBox.close()

        lineEdit.selectAll()
        lineEdit.setFocus()

def checkDirExist(lineEdit, qwidget):
    if not lineEdit.text(): return
    dirname = str( lineEdit.text() )
    if not os.path.isdir( dirname ):
        msg = "[ {0} ] \n Not found !!!".format(dirname)
        msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg,
                                                    QMessageBox.Ok, qwidget)
        msgBox.exec_()
        msgBox.close()

        lineEdit.selectAll()
        lineEdit.setFocus()

def showWarningMessageBox(msg, qwidget):
    msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg, QMessageBox.Ok, qwidget)
    msgBox.exec_()
    msgBox.close()



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import numpy as np
import re, sys, os, shutil
import tarfile
import subprocess as sub
import string, random, tempfile
import json
import logging
import gzip
import h5py

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

from . import ccmapHelpers as cmh

dtype_npBINarray = 'float32'

logger = logging.getLogger('ccmap')
logger.setLevel(logging.INFO)

class MapNotFoundError(Exception):
	def __init__(self, value):
		super(MapNotFoundError, self).__init__()
		self.value = value

	def __str__(self):
		return repr(self.value)

class ResolutionNotFoundError(Exception):
	def __init__(self, value):
		super(ResolutionNotFoundError, self).__init__()
		self.value = value

	def __str__(self):
		return repr(self.value)

def resolutionToBinsize(resolution):
	"""Return the bin size from the resolution unit

	It is a convenient function to convert resolution unit to binsize.
	It has a support of base (b), kilobase (kb), megabase (mb) and
	gigabase (gb) unit. It also convert decimal resolution unit as shown below
	in examples.

	Parameters
	----------

	resolution:	str
		resolution in b, kb, mb or gb.

	Returns
	-------
	binsize : int
		bin size

	Examples
	--------
		>>> resolutionToBinsize('1b')
		1
		>>> resolutionToBinsize('10b')
		10
		>>> resolutionToBinsize('1kb')
		1000
		>>> resolutionToBinsize('16kb')
		16000
		>>> resolutionToBinsize('1.23kb')
		1230
		>>> resolutionToBinsize('1.6mb')
		1600000
		>>> resolutionToBinsize('1.457mb')
		1457000

	"""
	factor = None
	base = None
	if re.search('\d+b', resolution) is not None:
		factor = 1
	elif re.search('kb', resolution) is not None:
		factor = 1000
	elif re.search('mb', resolution) is not None:
		factor = 1000000
	elif re.search('gb', resolution) is not None:
		factor = 1000000000
	else:
		raise ValueError(' \'{0}\' keyword of resolution not implemented.' .format(resolution))

	if re.search('(\d+.\d+)', resolution) is not None:
		base = float( re.search('(\d+.\d+)', resolution).group() )
	else:
		base = int( re.search('(\d+)', resolution).group() )

	return int(base*factor)

def binsizeToResolution(binsize):
	"""Return the resolution unit from the bin size

	It is a convenient function to convert binsize into resolution unit.
	It has a support of base (b), kilobase (kb), megabase (mb) and
	gigabase (gb) unit. It also convert binsize to decimal resolution unit as
	shown below in examples.

	Parameters
	----------

	binsize:	int
		bin size

	Returns
	-------
	resolution : str
		resolution unit

	Examples
	--------
		>>> binsizeToResolution(1)
		'1b'
		>>> binsizeToResolution(10)
		'10b'
		>>> binsizeToResolution(10000)
		'10kb'
		>>> binsizeToResolution(100000)
		'100kb'
		>>> binsizeToResolution(125500)
		'125.5kb'
		>>> binsizeToResolution(1000000)
		'1mb'
		>>> binsizeToResolution(1634300)
		'1.6343mb'

	"""

	suffix = { 0:'b', 1: 'kb', 2: 'mb', 3: 'gb' }
	rest = None
	count = 0
	while (1):
		rest = binsize/(1000**count)
		if rest >= 1000:
			count += 1
		else:
			break

	if rest - int(rest) == 0:
		resolution = str(int(rest)) + suffix[count]
	else:
		resolution = str(rest) + suffix[count]

	return resolution

class CCMAP:
	"""This class contains variables to store Hi-C Data.

	The class is instantiated by two methods:
		>>> ccMapObj = gcMapExplorer.lib.ccmap.CCMAP()
		>>> ccMapObj = gcMapExplorer.lib.ccmap.CCMAP(dtype='float32')

	Parameters
	----------
	dtype : str, Optional
			Data type for matrix. [``Default='float32'``]


	Attributes
	----------

	path2matrix : str
		Path to numpy array binary file on local file system
	yticks		:	list
		Minimum and maximum locations along Y-axis. e.g. ``yticks=[0, 400000]``
	xticks		:	list
		Minimum and maximum locations along X-axis. e.g. ``xticks=[0, 400000]``
	binsize		:  int
		Resolution of data. In case of 10kb resolution, binsize is 10000.
	title		:	str
		Title of the data
	xlabel		:	str
		Title for X-axis
	ylabel		:	str
		Title for Y-axis
	shape		:	tuple
		Overall shape of matrix
	minvalue	:	float
		Minimum value in matrix
	maxvalue	:	float
		Maximum value in matrix
	matrix		: 	numpy.memmap
		A memmap object pointing to matrix.

		HiC map data is saved as a numpy array binary file on local file system. This file can be only read after mapping to a numpy memmap object. After mapping, ``matrix``
		can be used as a numpy array.
		The file name is randomly generated as ``npBinary_XXXXXXXXXX.tmp``, where ``X`` can be a alphanumeric charecter. Please see details in |numpy memmap|.

		When ccmap is saved, this file is renamed with '.npbin' extension.

	bNoData		:	numpy.ndarray
		A boolean numpy array of matrix shape
	bLog		:	bool
		If values in matrix are in log
	state       :	str
		State of CCMAP object

		This keyword stores the state of the object. The state ensures when the numpy array binary file should be deleted from the local file system.

		Three keywords are used:
			* ``temporary`` :
				When object is created, it is in temporary state. After executing the script, numpy array binary file is automatically deleted from the local file system.

			* ``saved``:
				When a temporary or new object is saved, the numpy array binary file is copied to the destination directory and state is changed to saved.
				However, after saving, state become temporary. This method ensures that the saved copy is not deleted and only temporary copy is deleted after executing the script.

				When a already saved object is loaded, it is in saved state. The numpy array binary file is read from the original location and remaines saved
				at the original location after executing the script.

			* ``compressed``:
				When object is saved and numpy array binary file is simultaneously compressed, it is saved as compressed state. When this object is loaded,
				the numpy array binary file is decompressed into the working directory. This decompressed file is automatically deleted after
				execution of script while compressed file remains saved at the original location.

	dtype       :	str
		Data type of matrix

	"""

	def __init__(self, dtype=dtype_npBINarray):
		self.path2matrix = ''
		self.xticks = None
		self.yticks = None
		self.binsize = 0
		self.title = None
		self.xlabel = None
		self.ylabel = None
		self.shape = None
		self.minvalue = 9e10
		self.maxvalue = -9e10
		self.bNoData = None
		self.bLog = False
		self.dtype = dtype
		self.matrix = None

		self.state = 'temporary'

	def __del__(self):
		if self.matrix is not None:
			self.matrix._mmap.close()
		if self.matrix is not None:
			self.matrix
		if self.state == 'temporary' or self.state == 'compressed':
			if os.path.exists(self.path2matrix):
				os.remove(self.path2matrix)
		del self.xticks
		del self.yticks
		del self.shape

	def get_ticks(self, binsize=None):
		"""To get xticks and yticks for the matrix

		Parameters
		----------
		binsize : int
			Number of base in each bin or pixel or box of contact map.

		Returns
		-------
		xticks	:	numpy.array
			1D array containing positions along X-axis
		yticks	:	numpy.array
			1D array containing positions along X-axis

		"""
		if binsize is None:
			xticks = np.arange(self.xticks[0], self.xticks[1], self.binsize)
			yticks = np.arange(self.yticks[0], self.yticks[1], self.binsize)
		else:
			yticks = np.arange(self.yticks[0], self.yticks[1], binsize)
			xticks = np.arange(self.xticks[0], self.xticks[1], binsize)
		return xticks, yticks

	def gen_matrix_file(self, workDir=None):
		# Working and output directory
		if workDir is None:
			workDir = config['Dirs']['WorkingDirectory']

		# Only filename is generated
		fd, self.path2matrix = tempfile.mkstemp(prefix='npBinary_', suffix='.tmp', dir=workDir)
		os.close(fd)
		try:
			os.remove(self.path2matrix)
		except:
			pass

	def make_readable(self):
		"""Enable reading the numpy array binary file.

		Matrix file is saved on local file system. This file can be only read after mapping to a memmap object. This method maps the numpy memmap object to self.matrix variable.
		After using this method, :attr:`gcMapExplorer.lib.ccmap.CCMAP.matrix` can be used directly as similar to numpy array. Please see details in |numpy memmap|

		"""
		if self.matrix is not None:
			del self.matrix
		self.matrix = np.memmap(self.path2matrix, dtype=self.dtype, mode='r', shape=self.shape)

	def make_unreadable(self):
		"""Disable reading the numpy array binary file from local file system

		"""
		if self.matrix is not None:
			del self.matrix
			self.matrix = None

	def make_writable(self):
		"""Create new numpy array binary file on local file system and enable reading/writing to this file

		.. note::
		 	If a matrix file with similar name is already present, old file will be backed up.

		"""
		if self.matrix is not None:
			del self.matrix
		if os.path.exists(self.path2matrix):
			gen_backup(self.path2matrix)
		self.matrix = np.memmap(self.path2matrix, dtype=self.dtype, mode='w+', shape=self.shape)

	def make_editable(self):
		"""Enable editing numpy array binary file

		"""
		if self.matrix is not None:
			del self.matrix
		self.matrix = np.memmap(self.path2matrix, dtype=self.dtype, mode='r+', shape=self.shape)

	def copy(self, fill=None):
		"""To create a new copy of CCMAP object

		This method can be used to create a new copy of :class:`gcMapExplorer.lib.ccmap.CCMAP`.
		A new numpy array binary file will be created and all values from old file will be copied.

		Parameters
		----------
		fill : float
			Fill map with the value. If not given, map values will be copied.

		"""

		ccMapObj = CCMAP(dtype=self.dtype)

		for key in self.__dict__:
			ccMapObj.__dict__[key] = self.__dict__[key]

		# Generaing temporary numpy array file
		dirname = os.path.dirname( self.path2matrix )
		ccMapObj.gen_matrix_file(workDir=dirname)

		#ccMapObj.path2matrix = os.path.join(os.getcwd(), 'nparray_' + name_generator() + '.bin')

		self.make_readable()
		ccMapObj.make_writable()

		if fill is None:
			ccMapObj.matrix[:] = self.matrix[:]
		else:
			ccMapObj.matrix.fill(fill)

		ccMapObj.matrix.flush()
		self.make_unreadable()
		ccMapObj.make_unreadable()

		return ccMapObj

def jsonify(ccMapObj):
	"""Changes data type of attributes in CCMAP object for |json link|.

	Before saving the CCMAP object, its attributes data types are neccessary to change because few data types are not supported by json.

	Therefore, it is converted into other data types which are supported by json.
	These are the following attributes which are changed:

	+-------------+----------------------------------+----------------------------+
	| Attributes  | Original                         | modified                   |
	+-------------+----------------------------------+----------------------------+
	| bNoData     | numpy boolean array              | string of 0 and 1          |
	+-------------+----------------------------------+----------------------------+
	| xticks      | list of integer                  | list of string             |
	+-------------+----------------------------------+----------------------------+
	| yticks      | list of integer                  | list of string             |
	+-------------+----------------------------------+----------------------------+
	| minvalue    | float                            | string                     |
	+-------------+----------------------------------+----------------------------+
	| maxvalue    | float                            | string                     |
	+-------------+----------------------------------+----------------------------+
	| shape       | tuple of integer                 | list of string             |
	+-------------+----------------------------------+----------------------------+


	..	warning::
		If a object is passed through this method, it should be again passed through :meth:`gcMapExplorer.lib.ccmap.dejsonify` for any further use.
		Otherwise, this object cannot be used in any other methods because of the attributes data type modifications.


	Parameters
	----------
	ccMapObj	:	:class:`gcMapExplorer.lib.ccmap.CCMAP`
		A CCMAP object

	Returns
	-------
	None



	"""
	if ccMapObj.bNoData is not None:
		ccMapObj.bNoData = ''.join(list(map(str, list(np.array(ccMapObj.bNoData).astype(int)) )))

	if ccMapObj.xticks is not None:
		ccMapObj.xticks = list(map(str, ccMapObj.xticks))

	if ccMapObj.yticks is not None:
		ccMapObj.yticks = list(map(str, ccMapObj.yticks))

	ccMapObj.minvalue = str(float(ccMapObj.minvalue))
	ccMapObj.maxvalue = str(float(ccMapObj.maxvalue))
	ccMapObj.shape = list(map(str, ccMapObj.shape))

def dejsonify(ccMapObj, json_dict=None):
	"""Change back the data type of attributes in CCMAP object.

	Before loading the CCMAP object, its attributes data types are neccessary to change back.

	Therefore, it is converted into original data types as shown in a table (see :meth:`gcMapExplorer.lib.ccmap.jsonify`)

	Parameters
	----------
	ccMapObj	:	:class:`gcMapExplorer.lib.ccmap.CCMAP`
		A CCMAP object

	json_dict	:	dict, Optional
		A directory obtained after loading the saved file through json.

	"""
	if json_dict is not None:
		for key in json_dict:
			ccMapObj.__dict__[key] = json_dict[key]

	if ccMapObj.bNoData is not None:
		ccMapObj.bNoData = np.array( list(map(int, list(ccMapObj.bNoData))), dtype=bool)

	if ccMapObj.xticks is not None:
		ccMapObj.xticks = list(map(int, ccMapObj.xticks))

	if ccMapObj.yticks is not None:
		ccMapObj.yticks = list(map(int, ccMapObj.yticks))

	ccMapObj.minvalue = float(ccMapObj.minvalue)
	ccMapObj.maxvalue = float(ccMapObj.maxvalue)
	ccMapObj.shape = tuple(map(int, ccMapObj.shape))

def save_ccmap(ccMapObj, outfile,  compress=False, logHandler=None):
	""" Save CCMAP object on file

	CCMAP object can be saved as file for easy use. |json link| is used to save the object.
	The binary numpy array file is copied in the destination directory. If ``compress=True``, the array file will be compressed in gzip format.

	.. note::
		* Compression significantly reduces the array file size. However, its loading is slow during initiation when file is decompressed.

		* After loading, the decompressed binary numpy array file takes additional memory on local file system.




	Parameters
	----------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		A CCMAP object, which has to be saved
	outfile : str
		Name of output file including path to the directory/folder where file should be saved.
	compress	:	bool
		If ``True``, numpy array file will be compressed.

	Returns
	-------
	None

	"""

	# logger
	logHandler = logHandler
	logger = logging.getLogger('save_ccmap')
	if logHandler is not None:
		logger.propagate = False
		logger.addHandler(logHandler)
	logger.setLevel(logging.INFO)

	ccMapObj.make_unreadable()

	# Get file extension of output file name. If output name does not contain .ccmap extension it will be added
	file_extension = os.path.splitext(outfile)[1]
	if file_extension != '.ccmap':
		outfile = outfile + '.ccmap'

	# Getting output directory and generating path name for numpy array file
	outdir = os.path.dirname( os.path.abspath( os.path.expanduser(outfile)) )
	basename = os.path.basename( outfile )

	nparrayFileName = os.path.splitext(basename)[0] + '.npbin'
	nparrayfile = os.path.join(outdir,  nparrayFileName)

	logger.info(" Saving ccmap to file [{0}] and [{1}] ..." .format(outfile, nparrayfile))
	# Copying numpy array file to the output directory
	if ccMapObj.path2matrix == nparrayfile:
		logging.warning('File {0} already exists !!! If you have used make_writable or make_editable, this file is already modified and saved...' .format(ccMapObj.path2matrix))
	else:
		# Generating backup of old numpy array file
		if os.path.exists(nparrayfile):
			gen_backup(nparrayfile)
		shutil.copy(ccMapObj.path2matrix, nparrayfile)

	# Generating .ccmap backup file if it is already present in the output directory
	if os.path.exists(outfile):
		gen_backup(outfile)

	# Only store numpy array file in object because this file and .ccmap file should be present in same directory
	t_path2matrix = ccMapObj.path2matrix
	ccMapObj.path2matrix = nparrayFileName

	# converting data type before saving .ccmap
	jsonify(ccMapObj)
	if compress:
		ccMapObj.state = 'compressed'
	else:
		ccMapObj.state = 'saved'

	# saving .ccmap
	fout =  open( os.path.abspath( os.path.expanduser(outfile)), "w" )
	json.dump(ccMapObj.__dict__, fout, indent=4, separators=(',', ':'))
	fout.close()

	logger.info(" Compressing [{0}] ..." .format(nparrayfile))

	# Compressing numpy array file and removing original file
	if compress:

		if os.path.exists(nparrayfile+'.gz'):
			gen_backup(nparrayfile+'.gz')

		with open(nparrayfile, 'rb') as f_in:
			with gzip.open(nparrayfile + '.gz', 'wb') as f_out:
				shutil.copyfileobj(f_in, f_out)

		if os.path.exists(nparrayfile):
			os.remove(nparrayfile)

	logger.info("       Finished!!!\n")

	# Reverting the path of matrix file so that this object can be still used
	ccMapObj.path2matrix = t_path2matrix
	ccMapObj.state = 'temporary'
	dejsonify(ccMapObj)

def load_ccmap(infile, workDir=None):
	""" Load CCMAP object from an input file

	CCMAP object can be created from the input file, which was earlier saved using :meth:`gcMapExplorer.lib.ccmap.save_ccmap`.
	If the binary numpy array is compressed, this file is automatically extracted in the current working directory. After completion of the execution,
	this decompressed file will be automatically deleted. The compressed saved file will be remained unchanged.

	Parameters
	----------
	infile : str
		Name of the inputp file including path to the directory/folder where file is saved.

	workDir : str
		Name of working directory, where temporary files will be kept.If ``workDir = None``, file will be generated in OS based temporary directory.

	Returns
	-------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		A CCMAP object

	"""

	# Working directory
	if workDir is None:
		workDir = config['Dirs']['WorkingDirectory']

	# New object
	ccMapObj = CCMAP()

	# Open .ccmap file and load all keywords
	fin = open( infile, "r" )
	json_dict = json.load( fin )
	fin.close()

	# Generating the object from the json dictionary
	dejsonify(ccMapObj, json_dict)

	# Generating full path to numpy array file
	indir = os.path.dirname( os.path.abspath( os.path.expanduser(infile) ) )
	nparrayInFile = os.path.join( indir, ccMapObj.path2matrix)

	# if numpy array file is compressed, extract it and move to current working directory
	# else use this array file directly from original location
	if os.path.exists(nparrayInFile + '.gz'):
		ccMapObj.gen_matrix_file(workDir=workDir)      # Generate matrix file

		# try to catch system-exit and keyboard intrrupt from user. Have to delete manually created file
		try:
			with gzip.open(nparrayInFile + '.gz', 'rb') as f_in:
				with open(ccMapObj.path2matrix, 'wb') as f_out:
					shutil.copyfileobj(f_in, f_out)
		except (KeyboardInterrupt, SystemExit) as e:
			os.remove(ccMapObj.path2matrix)
			raise e

		# Making this obeject as temorary
		ccMapObj.state = 'temporary'
	else:
		ccMapObj.path2matrix = nparrayInFile

	return ccMapObj

def export_cmap(cmap, outfile, doNotWriteZeros=True):
	"""To export ``.ccmap`` as text file

	This function export ``.ccmap`` as coordinate list (COO) format sparse matrix file.
	In COO format, lists of (row, column, value) as three tab seprated columns are written in output file.

	Parameters
	----------
	ccmap : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		An instance of :class:`gcMapExplorer.lib.ccmap.CCMAP`, which is need to be exported.
	outfile : str
		Output file name.
	doNotWriteZeros : bool
		Do not write Zero values. It reduces memory of file.

	"""

	fout = open(outfile, 'w')

	if hasattr(cmap, make_readable):
		ccmap.make_readable()

	try:
		for i in range(ccmap.shape[0]):
			for j in range(i+1):
				if doNotWriteZeros:
					if ccmap.matrix[i][j] != 0:
						fout.write('{0}\t{1}\t{2}\n'.format(i, j, ccmap.matrix[i][j] ))
					else:
						fout.write('{0}\t{1}\t{2}\n'.format(i, j, ccmap.matrix[i][j] ))

	except (SystemExit, KeyboardInterrupt) as e:
		del ccmap
		raise e

def downSampleCCMap(cmap, level=2, workDir=None):

	level = int(level)
	# Working and output directory
	if workDir is None:
		workDir = config['Dirs']['WorkingDirectory']

	# To determine output shape but it does not contain remainder regions
	# subtracted one as real data start from first index
	outShapeX = int( np.ceil(float(cmap.shape[1] - 1 )/level) )
	outShapeY = int( np.ceil(float(cmap.shape[0] - 1 )/level) )

	# To determine remainder regions(sourceMasked.max() - sourceMasked.min())
	pad_size_outer = int( ( outShapeY * level) - (cmap.shape[0] - 1) )
	pad_size_inner = int( ( outShapeX * level) - (cmap.shape[1] - 1) )

	# Otput shape
	outputShape = (outShapeX + 1 , outShapeY + 1)

	#rint(pad_size_outer, pad_size_inner, outputShape)

	outCmap = cmp.CCMAP(dtype=np.float)
	outCmap.shape = outputShape
	outCmap.gen_matrix_file(workDir=workDir)
	outCmap.make_writable()

	# Ignore first row and column --- no data
	count = 1
	i = 1
	while i < cmap.shape[0]:

	    # Here handles inner
	    if pad_size_inner != 0:
	        zero_part = []
	        for n in range(level):
	            zero_part.append( np.zeros(pad_size_inner) )

	        # Here handles outer
	        if i+level < cmap.shape[0]:
	            x_padded = np.hstack( (cmap.matrix[i:i + level, 1:], zero_part) )
	        else:
	            x_padded = np.hstack( (cmap.matrix[i:, 1:], zero_part[:cmap.matrix[i:].shape[0]] ) )

	        # print(x_padded.shape)

	        r = x_padded.sum(axis=0).reshape(-1, level).sum(axis=1)

	        #rint(r.shape)

	    else:

	        # Here handles outer
	        if i+level < cmap.shape[0]:
	            r = cmap.matrix[i:i+level, 1:].sum(axis=0).reshape(-1, level).sum(axis=1)
	        else:
	            remain = cmap.shape[0]-i
	            r = cmap.matrix[i:, 1:].sum(axis=0).reshape(-1, remain).sum(axis=1)

	    i = i + level

	    outCmap.matrix[count, 1:] = r
	    count = count + 1

	outCmap.matrix.flush()

	# Assign minimum value
	ma = np.ma.masked_equal(outCmap.matrix, 0.0, copy=False)
	outCmap.minvalue = ma.min()
	del ma

	# Assign Maximum value
	outCmap.maxvalue = np.amax(outCmap.matrix)

	# New binsize
	outCmap.binsize = cmap.binsize * level

	# xlabel and ylabel
	outCmap.xlabel = cmap.xlabel
	outCmap.ylabel = cmap.ylabel

	return outCmap

def downSample1D(array, level=2, func='max'):
	""" Downsample or coarse a one-dimensional array
	"""

	level = int(level)
	outShape = int( np.ceil(float(array.shape[0] - 1 )/level) )
	pad_size = int( ( outShape * level) - (array.shape[0] - 1) )

	outputShape = outShape + 1

	outputArray = np.zeros(outputShape, dtype=array.dtype)
	if pad_size != 0:
		x_padded = np.hstack( (array[1:], np.zeros(pad_size)) )
	else:
		x_padded = array[1:]


	if func =='sum':
		outputArray[1:] = x_padded.reshape(-1, level).sum(axis=1)
	elif func =='mean':
		outputArray[1:] = x_padded.reshape(-1, level).mean(axis=1)
	else:
		outputArray[1:] = x_padded.reshape(-1, level).max(axis=1)

	return outputArray


def smoothen_map(ccMapObj, filter_pass=1):
	""" Smoothen map using unweighted sliding-average smooth method.

	This function can be used to smooth out the map. It implements unweighted sliding-average smooth method.

	Parameters
	----------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		A CCMAP object, which has to be smoothen.
	filter_pass : int
		Number of pass to smooth. Larger the number of pass, more smoother is the map. However, pattern in maps may be difficult to identify when
		pass number is high.

	Returns
	-------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		The smoothed CCMAP.

	"""

	smooth_map = ccMapObj.copy()
	smooth_map.make_editable()
	cmh.make_smooth_map(smooth_map.matrix, smooth_map.shape, filter_pass=filter_pass)
	smooth_map.make_unreadable()
	return smooth_map

def GetTransitionProbablityMatrix(ccMapObj,  percentile_thershold_no_data=None, thershold_data_occup=None):
	""" To calculate transition probablity matrix.

	This method can be used to calculate transition probablity matrix. This is similar to markov-chain transition matrix.

	:note: This transition matrix is not symmetric, because each row represents stochastic row vector,
		which contains contact probablity of this bin with every other bins and sum of row is always equal to one. See here: |markov chain link|.

	Parameters
	----------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		A CCMAP object, which has to be smoothen.

	percentile_thershold_no_data : int
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		``percentile_thershold_no_data`` should be between 1 and 100. This options discard the rows and columns which are above this percentile.
		For example: if this value is 99, those row or columns will be discarded which contains larger than number of zeros (missing data) at 99 percentile.

		To calculate percentile, all blank rows are removed, then in all rows, number of zeros are counted. Afterwards, number of zeros at
		`percentile_thershold_no_data` percentile is obtained. In next step, if a row contain number of zeros larger than this percentile value,
		the whole row and column is assigned to have missing data. This percentile indicates highest numbers of zeros (missing data) in given rows/columns.

	thershold_data_occup : float
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		This ratio is (number of bins with data) / (total number of bins in the given row/column).
		For example: if `thershold_data_occup = 0.8`, then all rows containing more than 20\% of missing data will be discarded.

		Note that this parameter is suitable for low resolution data because maps are likely to be much less sparse.

	Returns
	-------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		The smoothed CCMAP.

	"""

	NormHiCmap = ccMapObj.copy()
	NormHiCmap.make_editable()
	ccMapObj.make_readable()

	bNonZeros = cmh.get_nonzeros_index(ccMapObj.matrix, thershold_percentile=percentile_thershold_no_data, thershold_data_occup=thershold_data_occup)
	A = (ccMapObj.matrix[bNonZeros,:])[:,bNonZeros]   # Selected row-column which are not all zeros

	NormHiCmap.minvalue, NormHiCmap.maxvalue = cmh.CalculateTransitionProbablity(ccMapObj.matrix, ~bNonZeros, Out=NormHiCmap.matrix)

	NormHiCmap.bNoData = ~bNonZeros
	NormHiCmap.matrix=None

	return NormHiCmap

def LogOfMatrix(ccMapObj):

	ccMapObj.make_readable()

	LogHiCmap = CCMAP()
	LogHiCmap.path2matrix = os.getcwd() + '/nparray_' + name_generator() + '.bin'

	LogHiCmap.shape = ccMapObj.shape
	LogHiCmap.xticks = ccMapObj.xticks
	LogHiCmap.yticks = ccMapObj.yticks
	LogHiCmap.binsize = ccMapObj.binsize
	LogHiCmap.bLog = True

	bNonZeros = None
	#if ccMapObj.bNoData is not None:
	#	LogHiCmap.bNoData = ccMapObj.bNoData
	#	bNonZeros = ~LogHiCmap.bNoData
	#else:
	LogHiCmap.bNoData = np.all( ccMapObj.matrix == 0.0, axis=0)
	bNonZeros = ~LogHiCmap.bNoData

	# Log of part of matrix containing data
	path2matrixA = os.getcwd() + '/nparray_' + name_generator() + '.bin'
	A = (ccMapObj.matrix[bNonZeros,:])[:,bNonZeros]   # Selected row-column which are not all zeros
	BinMatrixA = np.memmap(path2matrixA, dtype=dtype_npBINarray, mode='w+', shape=A.shape)
	BinMatrixA[:] = np.log10(A)[:]
	BinMatrixA.flush()

	# Assigning minvalue and maxvalue
	LogHiCmap.maxvalue = float(np.amax(BinMatrixA))
	minvalue = np.amin(BinMatrixA)
	v_steps = np.linspace(minvalue, LogHiCmap.maxvalue, 100)
	LogHiCmap.minvalue = minvalue - (v_steps[1] - v_steps[0])

	# Making full matrix
	BinLogMatrix = np.memmap(LogHiCmap.path2matrix, dtype=dtype_npBINarray, mode='w+', shape=LogHiCmap.shape)
	A_i = -1
	A_j = 0
	for i in range(BinLogMatrix.shape[0]):
		if not LogHiCmap.bNoData[i]:
			A_i += 1

		A_j = 0
		for j in range(BinLogMatrix.shape[1]):
			if LogHiCmap.bNoData[i] or LogHiCmap.bNoData[j]:
				BinLogMatrix[i][j] = LogHiCmap.minvalue
			else:
				BinLogMatrix[i][j] = BinMatrixA[A_i][A_j]
				A_j += 1
	BinLogMatrix.flush()

	del BinLogMatrix
	del BinMatrixA

	try:
		os.remove(path2matrixA)
	except:
		pass

	return LogHiCmap

def StationaryDistributionByMultiply(ccMapObj, stop_tol=1e-12, vector=None, iteration=None):

	ccMapObj.make_readable()
	bNonZeros = ~ccMapObj.bNoData
	mem = psutil.virtual_memory()

	if ccMapObj.matrix.nbytes < mem.available:
		print("Using RAM")
		vector = cmh.CalcStationaryDistributionRAM(ccMapObj.matrix, ccMapObj.shape, bNonZeros, stop_tol=stop_tol, iteration=None)
	else:
		print("Using HDD")
		vector = cmh.CalcStationaryDistributionHDD(ccMapObj.matrix, ccMapObj.shape, bNonZeros, stop_tol=stop_tol, iteration=None)

	return vector

def StationaryDistributionByEigenvector(ccMapObj):

	ccMapObj.make_readable()
	prob_eigenvector = cmh.CalcStationaryDistributionByEigen(ccMapObj.matrix, ccMapObj.shape, ccMapObj.bNoData)
	ccMapObj.make_unreadable()

	return prob_eigenvector

def gen_HiC_svg_plot(ccMapObj, outfile):

	import matplotlib
	matplotlib.use('SVG')
	import matplotlib.pyplot as plt

	# File namings
	outdir = os.path.dirname( os.path.abspath(outfile) )
	outfilebase = os.path.splitext(os.path.basename(os.path.abspath(outfile)))[0]
	####

	def make_plot(idx, jdx, mat, i, j, minvalue, maxvalue, idiv=2000, jdiv=10000):
		fig = plt.figure(111)

		plt.imshow(mat[i:i+idiv, j:j+jdiv], origin='lower', aspect='equal', cmap='Greys', interpolation='none', vmax=maxvalue, vmin=minvalue)

		ax = plt.gca()

		plt.tick_params(axis='x', which='both', bottom='off', top='off', labelbottom='off')
		ax.spines['top'].set_visible(False)
		ax.spines['bottom'].set_visible(False)

		name = os.path.join(outdir, '{0}.{1}.{2}.{3}.{4}.svg' .format(outfilebase, idx, jdx, idiv, jdiv))

		plt.savefig(name)
		plt.clf()
		del fig

	def combine_svg(data):
		fout = open(outfile, 'w')
		fout.write('<?xml version="1.0" encoding="utf-8" standalone="no"?>\n')
		fout.write('<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">\n')
		fout.write('<svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"> \n')
		fout.write('<g id="figure_1">\n')

		y = 0
		idx = 1
		for i in range(len(data)-1, -1, -1):
			x = 0
			tmp_y = 0
			for j in range(len(data[i])):
				filename = os.path.join( outdir, '{0}.{1}.{2}.{3}.{4}.svg' .format(outfilebase, i, j, data[i][j][0], data[i][j][1]))
				tmp_y = data[i][j][0]

				fin = open(filename, 'r')
				read_png = False
				png_data = ''
				for line in fin:
					if re.search('iVBORw0', line):
						png_data = line.split('"')[0]
						break
				fin.close()

				fout.write('<image x="{0}" xlink:href="data:image{1}/png;base64, \n' .format(x, idx))
				fout.write(png_data)
				fout.write('" y="{0}" transform="scale(0.1)" />\n' .format(y))

				x += data[i][j][1]

				os.remove(filename)

			y += tmp_y

		fout.write('</g>\n')
		fout.write('</svg>\n')
		fout.close()

	# Main part of function start here
	ccMapObj.make_readable()

	# Minimum and maximum value
	minvalue = ccMapObj.minvalue
	maxvalue = ccMapObj.maxvalue

	print (minvalue, maxvalue)

	idiv = 1000
	jdiv = 10000
	i = 0
	idx = 0
	file_data = []

	print("Generating plot segements as svg files ...\n")
	while(1):
		outer_loop_finished = False
		j = 0
		jdx = 0
		tmp_file_data = []
		while(1):
			if i+idiv >= ccMapObj.shape[0] and j+jdiv >=ccMapObj.shape[0] :
				make_plot(idx, jdx, ccMapObj.matrix, i, j, minvalue, maxvalue, idiv=ccMapObj.shape[0]-i, jdiv=ccMapObj.shape[0]-j)
				outer_loop_finished = True
				tmp_file_data.append([ccMapObj.shape[0]-i, ccMapObj.shape[0]-j])
				break

			elif i+idiv >= ccMapObj.shape[0] and j+jdiv < ccMapObj.shape[0] :
				make_plot(idx, jdx, ccMapObj.matrix, i, j, minvalue, maxvalue, idiv=ccMapObj.shape[0]-i, jdiv=jdiv)
				tmp_file_data.append([ccMapObj.shape[0]-i, jdiv])

			elif i+idiv < ccMapObj.shape[0] and j+jdiv >= ccMapObj.shape[0] :
				make_plot(idx, jdx, ccMapObj.matrix, i, j, minvalue, maxvalue, idiv=idiv, jdiv=ccMapObj.shape[0]-j)
				tmp_file_data.append([idiv, ccMapObj.shape[0]-j])
				break

			else:
				make_plot(idx, jdx, ccMapObj.matrix, i, j, minvalue, maxvalue, idiv=idiv, jdiv=jdiv)
				tmp_file_data.append([idiv, jdiv])

			j += jdiv
			jdx += 1

		file_data.append(tmp_file_data)

		if outer_loop_finished:
			break

		i += idiv
		idx += 1

	print("Merging plot segments in one svg file...\n")
	combine_svg(file_data)

def name_generator(size=10, chars=string.ascii_letters + string.digits ):
	 return ''.join(random.choice(chars) for _ in range(size))

def gen_backup(filename):
	fc = 1
	full_backup_path = ''
	indir = os.path.dirname( filename )
	basename = os.path.basename( filename )
	while 1:
		backup_filename = '#' + basename + '.{0}#' .format(fc)
		full_backup_path = os.path.join(indir,  backup_filename)
		if not os.path.exists(full_backup_path):
			break
		fc += 1

	shutil.move(filename, full_backup_path)
	logger.info('File {0} already exists !!! Backing up as {1} .' .format(filename, full_backup_path))

	return



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import numpy as np
from . import ccmapHelpers as cmh
from . import cmstats


def _normalizeByAvgContact(ccMapObj, normCCMap, stats='median', percentile_thershold_no_data=None, thershold_data_occup=None):

    normCCMap.make_editable()
    ccMapObj.make_readable()

    bNonZeros = cmh.get_nonzeros_index(ccMapObj.matrix, thershold_percentile=percentile_thershold_no_data, thershold_data_occup=thershold_data_occup)
    normCCMap.bNoData = ~bNonZeros

    ccMapObj.make_readable()
    avgContacts = cmstats.getAvgContactByDistance(ccMapObj, stats=stats)

    # Do along diagonal
    for m in range(ccMapObj.shape[0]):

        # do for postive diagonal offset
        idx = cmh.kth_diag_indices(m, ccMapObj.matrix)
        if avgContacts[m] != 0:
            normCCMap.matrix[idx] = ccMapObj.matrix[idx] / avgContacts[m]
        else:
            normCCMap.matrix[idx] = ccMapObj.matrix[idx]

        # Do for negative diagonal offset
        del idx
        if m != 0:
            idx = cmh.kth_diag_indices(m*-1, ccMapObj.matrix)
            if avgContacts[m] != 0:
                normCCMap.matrix[idx] = ccMapObj.matrix[idx] / avgContacts[m]
            else:
                normCCMap.matrix[idx] = ccMapObj.matrix[idx]
            del idx

    marray = np.ma.masked_equal(normCCMap.matrix, 0.0, copy=False)
    normCCMap.minvalue = marray.min()
    normCCMap.maxvalue = np.amax(normCCMap.matrix)

    return normCCMap
